#!/usr/bin/env python

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import vtk
from vmtk import vtkvmtk
from vmtk import pypes


class vmtkImageCompare(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None
        self.ReferenceImage = None
        self.Method = ''
        self.Tolerance = 1E-8
        self.Result = ''
        self.ResultLog = ''

        self.SetScriptName('vmtkimagecompare')
        self.SetScriptDoc('compares an image against a reference')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['ReferenceImage','r','vtkImageData',1,'','the reference image to compare against','vmtkimagereader'],
            ['Method','method','str',1,'["subtraction","range"]','method of the test'],
            ['Tolerance','tolerance','float',1,'','tolerance for numerical comparisons'],
            ])
        self.SetOutputMembers([
            ['Result','result','bool',1,'','Output boolean stating if images are equal or not'],
            ['ResultLog','log','str',1,'','Result Log']
            ])

    def rangeCompare(self):

        imageRange = self.Image.GetPointData().GetScalars().GetRange()
        referenceRange = self.ReferenceImage.GetPointData().GetScalars().GetRange()
        rangeDiff = (imageRange[0] - referenceRange[0], imageRange[1] - referenceRange[1])

        self.InputInfo('Image Range: '+ str(imageRange))
        self.InputInfo('Reference Image Range: '+ str(referenceRange))
        self.InputInfo('Range Difference: '+ str(rangeDiff))

        if max([abs(d) for d in rangeDiff]) < self.Tolerance:
            return True

        return False

    def subtractionCompare(self):

        imagePoints = self.Image.GetNumberOfPoints()
        referencePoints = self.ReferenceImage.GetNumberOfPoints()

        self.InputInfo('Image Points: ' + str(imagePoints))
        self.InputInfo('Reference Image Points: ' + str(referencePoints))

        if abs(imagePoints - referencePoints) > 0 :
            self.ResultLog = 'Uneven NumberOfPoints'
            return False

        imageScalarType = self.Image.GetScalarType()
        referenceScalarType = self.ReferenceImage.GetScalarType()
        if imageScalarType != referenceScalarType:
            self.PrintError('Error: Input image and reference image are not of \
                the same type. Please cast images to the same type.')

        imageMath = vtk.vtkImageMathematics()
        imageMath.SetInput1Data(self.Image)
        imageMath.SetInput2Data(self.ReferenceImage)
        imageMath.SetOperationToSubtract()
        imageMath.Update()
        differenceImage = imageMath.GetOutput()
        differenceRange = differenceImage.GetPointData().GetScalars().GetRange()

        self.InputInfo('Difference Range: ' + str(differenceRange))

        if max([abs(d) for d in differenceRange]) < self.Tolerance:
            return True

        return False

    def Execute(self):

        if not self.Image:
            self.PrintError('Error: No image.')
        if not self.ReferenceImage:
            self.PrintError('Error: No reference image.')
        if not self.Method:
            self.PrintError('Error: No method.')

        if (self.Method == 'subtraction'):
            self.Result = self.subtractionCompare()
        elif (self.Method == 'range'):
            self.Result = self.rangeCompare()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   PypeS
## Module:    $RCSfile: pype.py,v $
## Language:  Python
## Date:      $Date: 2006/07/07 10:45:42 $
## Version:   $Revision: 1.18 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.


from __future__ import print_function, absolute_import # NEED TO STAY AS TOP IMPORT
import sys
import os.path
from vmtk import pypes


class PypeWrapper(object):

    def __init__(self):

        self.Mode = 'slicer3'
        self.XMLDescription = ''
        self.PypeTitle = ''
        self.PypeDescription = ''
        self.Contributor = ''
        self.ModuleFileName = ''

        self.Arguments = None
        self.ScriptList = []
        self.ModulePipeArguments = []
        self.AllExposedMembers = []

        self.Indentation = '  '

    def ParseArguments(self):
        if '--help' in self.Arguments:
            print('hey!')
            return
        if '--pype' not in self.Arguments:
            print('complain!')
            return
        if '--mode' in self.Arguments:
            self.Mode = self.Arguments[self.Arguments.index('--mode')+1]
        if '--title' in self.Arguments:
            self.PypeTitle = self.Arguments[self.Arguments.index('--title')+1]
        if '--description' in self.Arguments:
            self.PypeDescription = self.Arguments[self.Arguments.index('--description')+1]
        if '--contributor' in self.Arguments:
            self.Contributor = self.Arguments[self.Arguments.index('--contributor')+1]
        if '--modulefile' in self.Arguments:
            self.ModuleFileName = self.Arguments[self.Arguments.index('--modulefile')+1]
        arguments = self.Arguments[self.Arguments.index('--pype')+1:]
        self.ModulePipeArguments = arguments[:]
        while '--pipe' in arguments:
            scriptSlice = arguments[:arguments.index('--pipe')]
            self.ScriptList.append([os.path.splitext(os.path.split(scriptSlice[0])[1])[0],scriptSlice[1:]])
            arguments = arguments[arguments.index('--pipe')+1:]
        scriptSlice = arguments[:]
        if not arguments:
            return
        self.ScriptList.append([os.path.splitext(os.path.split(scriptSlice[0])[1])[0],scriptSlice[1:]])

    def Execute(self):
        ind = self.Indentation
        self.XMLDescription = '<?xml version="1.0" encoding="utf-8"?>\n'
        self.XMLDescription += '<executable>\n'
        self.XMLDescription += ind + '<category>vmtk</category>\n'
        self.XMLDescription += ind + '<title>%s</title>\n' % (self.PypeTitle)
        self.XMLDescription += ind + '<description>%s</description>\n' % (self.PypeDescription)
        self.XMLDescription += ind + '<contributor>%s</contributor>\n' % (self.Contributor)

        self.AllExposedMembers = []
        for scriptNameAndArguments in self.ScriptList:
            self.XMLDescription += ind + '<parameters>\n'
            scriptName = scriptNameAndArguments[0]
            moduleName = scriptName
            scriptArguments = scriptNameAndArguments[1]
            try:
                exec('from vmtk import '+ moduleName)
            except ImportError:
                print('No module named ' + moduleName)
                break
            scriptObjectClassName = ''
            exec ('scriptObjectClassName =  '+moduleName+'.'+moduleName)
            self.XMLDescription += 2*ind + '<label>%s Parameters</label>\n' % (scriptObjectClassName)
            moduleScriptObjectClassName = moduleName+'.'+scriptObjectClassName
            scriptObject = 0
            exec ('scriptObject = '+moduleScriptObjectClassName+'()')
            scriptArguments = scriptNameAndArguments[1]
            exposedArgumentNames = [argument.split('@')[0] for argument in scriptArguments if '@' in argument[1:]]
            exposedArgumentChannels = [argument.split('@')[1] for argument in scriptArguments if '@' in argument[1:]]
            exposedArgumentOptions = [scriptArguments[scriptArguments.index(argument)-1][1:] for argument in scriptArguments if '@' in argument[1:]]
            exposedOptionsToNamesAndChannels = {}
            for i in range(len(exposedArgumentOptions)):
                exposedOptionsToNamesAndChannels[exposedArgumentOptions[i]] = [exposedArgumentNames[i], exposedArgumentChannels[i]]
            exposedMembers = []
            for member in scriptObject.InputMembers + scriptObject.OutputMembers:
                exec('member.MemberValue = scriptObject.'+member.MemberName)
                if member.OptionName in exposedOptionsToNamesAndChannels:
                    member.ExposedName = exposedOptionsToNamesAndChannels[member.OptionName][0]
                    member.ExposedChannel = exposedOptionsToNamesAndChannels[member.OptionName][1]
                    exposedMembers.append(member)
                    self.AllExposedMembers.append(member)
            for exposedMember in exposedMembers:
                memberXMLTag = ''
                memberXMLOptions = ''
                enumeration = exposedMember.GetRangeEnumeration()
                if exposedMember.MemberType == 'int':
                    memberXMLTag = 'integer'
                elif exposedMember.MemberType == 'float':
                    memberXMLTag = 'float'
                elif exposedMember.MemberType == 'str':
                    memberXMLTag = 'string'
                    if enumeration:
                        memberXMLTag += '-enumeration'
                elif exposedMember.MemberType == 'bool':
                    memberXMLTag = 'boolean'
                if exposedMember.MemberLength != 1:
                    memberXMLTag += '-vector'
                if exposedMember.MemberType == 'vtkImageData':
                    memberXMLTag = 'image'
                elif exposedMember.MemberType == 'vtkPolyData':
                    memberXMLTag = 'geometry'
                if exposedMember.ExposedChannel == 'point':
                    memberXMLTag = 'point'
                    if exposedMember.MemberLength == -1:
                        memberXMLOptions += 'multiple="true"'
                self.XMLDescription += 2*ind + '<%s>\n' % (memberXMLTag+' '+memberXMLOptions)
                self.XMLDescription += 3*ind + '<name>%s</name>\n' % (exposedMember.ExposedName)
                self.XMLDescription += 3*ind + '<longflag>%s</longflag>\n' % (exposedMember.ExposedName)
                self.XMLDescription += 3*ind + '<label>%s</label>\n' % (exposedMember.ExposedName)
                if exposedMember.MemberDoc:
                    self.XMLDescription += 3*ind + '<description>%s</description>\n' % (exposedMember.MemberDoc)
                if exposedMember.MemberValue not in [None, [], '']:
                    self.XMLDescription += 3*ind + '<default>%s</default>\n' % (str(exposedMember.MemberValue))
                if enumeration:
                    for element in enumeration:
                        self.XMLDescription += 3*ind + '<element>%s</element>\n' % (str(element))
                values = exposedMember.GetRangeValues()
                if values:
                    self.XMLDescription += 3*ind + '<constraints>\n'
                    if values[0] != None:
                        self.XMLDescription += 4*ind + '<minimum>%s</minimum>\n' % (str(values[0]))
                    if values[1] != None:
                        self.XMLDescription += 4*ind + '<maximum>%s</maximum>\n' % (str(values[1]))
                    if values[2] != None:
                        self.XMLDescription += 4*ind + '<step>%s</step>\n' % (str(values[2]))
                    self.XMLDescription += 3*ind + '</constraints>\n'
                if exposedMember.ExposedChannel in ['input','output']:
                    self.XMLDescription += 3*ind + '<channel>%s</channel>\n' % (exposedMember.ExposedChannel)
                self.XMLDescription += 2*ind + '</%s>\n' % (memberXMLTag)
            self.XMLDescription += ind + '</parameters>\n'
        self.XMLDescription += '</executable>\n'

        moduleFile = open(self.ModuleFileName,'w')
        moduleFile.write('#!/usr/bin/env python\n\n')
        moduleFile.write('xmlDescription = """')
        moduleFile.write(self.XMLDescription)
        moduleFile.write('"""\n')
        moduleFile.write('\n')
        moduleFile.write('pypeWrapperCommand = "%s"\n' % ' '.join(sys.argv))
        moduleFile.write('\n')
        moduleFile.write('import sys\n')
        moduleFile.write('if "--xml" in sys.argv:\n')
        moduleFile.write(self.Indentation+'print(xmlDescription\n)')
        moduleFile.write(self.Indentation+'sys.exit(0)\n')
        moduleFile.write('\n')
        moduleFile.write('if "--logo" in sys.argv:\n')
        moduleFile.write(self.Indentation+'sys.exit(0)\n')
        moduleFile.write('\n')
        moduleFile.write('import sys\n')
        moduleFile.write('if "--pypewrapper" in sys.argv:\n')
        moduleFile.write(self.Indentation+'print(pypeWrapperCommand\n)')
        moduleFile.write(self.Indentation+'sys.exit(0)\n')
        moduleFile.write('\n')

        substModulePipeArguments = []
        exposedMembersOrder = []
        for argument in self.ModulePipeArguments:
            if '@' in argument[1:]:
                substModulePipeArguments.append(argument.split('@')[0])
            else:
                substModulePipeArguments.append(argument)
        for exposedMember in self.AllExposedMembers:
            exposedMembersOrder.append(substModulePipeArguments.index(exposedMember.ExposedName))
            if exposedMember.ExposedChannel in ['input','output']:
                substModulePipeArguments[substModulePipeArguments.index(exposedMember.ExposedName)-1] += 'file'
            substModulePipeArguments[substModulePipeArguments.index(exposedMember.ExposedName)] = '%s'
        sortedExposedMembersOrder = exposedMembersOrder[:]
        sortedExposedMembersOrder.sort()
        allOrderedExposedMemberNames = []
        for position in sortedExposedMembersOrder:
            allOrderedExposedMemberNames.append(self.AllExposedMembers[exposedMembersOrder.index(position)].ExposedName)

        moduleFile.write('arguments = sys.argv[:]\n')
        moduleFile.write('\n')

        for exposedMember in self.AllExposedMembers:
            if exposedMember.MemberType == 'bool':
                moduleFile.write('%s = "0"\n' % exposedMember.ExposedName)
                moduleFile.write('if "--%s" in arguments:\n' % (exposedMember.ExposedName))
                moduleFile.write(self.Indentation+'%s = "1"\n' % (exposedMember.ExposedName))
                moduleFile.write(self.Indentation+'arguments.remove("--%s")\n' % exposedMember.ExposedName)
                moduleFile.write('%s = " ".join(%s.split(","))\n' % (exposedMember.ExposedName, exposedMember.ExposedName))
                moduleFile.write('\n')
            else:
                moduleFile.write('%s = ""\n' % exposedMember.ExposedName)
                moduleFile.write('while "--%s" in arguments:\n' % (exposedMember.ExposedName))
                moduleFile.write(self.Indentation+'index = arguments.index("--%s")\n' % (exposedMember.ExposedName))
                moduleFile.write(self.Indentation+'if index != len(arguments)-1 and "--" not in arguments[index+1]:\n')
                moduleFile.write(2*self.Indentation+'if %s:\n' % exposedMember.ExposedName)
                moduleFile.write(3*self.Indentation+'%s += ","\n' % exposedMember.ExposedName)
                moduleFile.write(2*self.Indentation+'%s += arguments[index+1]\n' % exposedMember.ExposedName)
                moduleFile.write(2*self.Indentation+'arguments.remove(arguments[index+1])\n')
                moduleFile.write(self.Indentation+'arguments.remove("--%s")\n' % exposedMember.ExposedName)
                moduleFile.write('%s = " ".join(%s.split(","))\n' % (exposedMember.ExposedName, exposedMember.ExposedName))
                moduleFile.write('\n')

        moduleFile.write('pipe = "%s" %% (%s)\n' % (' '.join(substModulePipeArguments),','.join(allOrderedExposedMemberNames)))

        moduleFile.write('\n')
        moduleFile.write('from vmtk import pypes\n')
        moduleFile.write('pypes.PypeRun(pipe)\n')
        moduleFile.write('\n')

        moduleFile.close()


if __name__=='__main__':
    pipeLumper = PypeWrapper()
    pipeLumper.Arguments = sys.argv
    pipeLumper.ParseArguments()
    pipeLumper.Execute()



## Module:    $RCSfile: vmtknumpytocenterlines.py,v $
## Language:  Python
## Date:      June 10, 2017
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo
##       The Jacobs Institute

from __future__ import absolute_import  # NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys
from vtk.numpy_interface import dataset_adapter as dsa

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes

try:
    import numpy as np
except ImportError:
    raise ImportError('Unable to Import vmtknumpytosurface module, numpy is not installed')


class vmtkNumpyToCenterlines(pypes.pypeScript):
    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.ArrayDict = None

        self.SetScriptName('vmtkNumpyToCenterlines')
        self.SetScriptDoc('Takes a nested python dictionary containing numpy arrays specifying vertex '
                          'points, associated scalar data, and cell data specifying line connectivity'
                          'and returns a VTK centerlines vtkPolyData file')
        self.SetInputMembers([
            ['ArrayDict','i','dict',1,'','the input array dictionary','vmtknumpyreader']])
        self.SetOutputMembers([
            ['Centerlines','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']])

    def Execute(self):

        polyData = vtk.vtkPolyData()
        wpPolyData = dsa.WrapDataObject(polyData)

        points = self.ArrayDict['Points']
        vtkPoints = dsa.numpyTovtkDataArray(points, 'Points')
        wpPolyData.Points = vtkPoints

        self.PrintLog('converting point data')
        for pointDataKey in self.ArrayDict['PointData'].keys():
            pointDataItem = self.ArrayDict['PointData'][pointDataKey]
            wpPolyData.PointData.append(pointDataItem, name=pointDataKey)

        self.PrintLog('converting cell data')
        for cellKey in self.ArrayDict['CellData'].keys():
            if cellKey == 'CellPointIds':
                cellDataArray = vtk.vtkCellArray()
                for cellId in self.ArrayDict['CellData']['CellPointIds']:
                    numberOfCellPoints = cellId.size
                    cellDataArray.InsertNextCell(numberOfCellPoints)
                    for cellPoint in cellId:
                        cellDataArray.InsertCellPoint(cellPoint)
                wpPolyData.VTKObject.SetLines(cellDataArray)
            else:
                cellDataItem = self.ArrayDict['CellData'][cellKey]
                wpPolyData.CellData.append(cellDataItem, name=cellKey)

        self.Centerlines = wpPolyData.VTKObject


if __name__ == '__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   PypeS
## Module:    pypemain.py
## Language:  Python

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

"""Entry point for the "vmtk" command installed by the PyPI package.

This provides the same behavior as the vmtk launcher script at the root of
the source tree (vmtk.py), as an importable function that can be declared
as a console_scripts entry point in setup.py:

  vmtk <pype>          execute a pype (e.g. "vmtk vmtkimageviewer -ifile image.vti")
  vmtk --file FILE     execute the pypes listed in FILE
  vmtk [--ui pad]      start the interactive PypePad user interface
  vmtk --ui console    start an interactive console
  vmtk --help          print usage information
"""

from __future__ import print_function, absolute_import

import sys


def main():
    from vmtk import pypes

    vmtkOptions = ['--help', '--ui', '--file']

    if len(sys.argv) > 1 and sys.argv[1] not in vmtkOptions:
        arguments = sys.argv[1:]
        print("Executing", ' '.join(arguments))
        pipe = pypes.Pype()
        pipe.ExitOnError = 0
        pipe.Arguments = arguments
        pipe.ParseArguments()
        pipe.Execute()
        return 0

    if len(sys.argv) > 1 and '--file' in sys.argv:
        fileindex = sys.argv.index('--file') + 1
        if fileindex < len(sys.argv):
            with open(sys.argv[fileindex], 'r') as inputfile:
                physicalLines = [line for line in inputfile.readlines()
                                 if line and line.strip() and line.strip()[0] != '#']
            lines = []
            for line in physicalLines:
                if lines and lines[-1].endswith('\\\n'):
                    lines[-1] = lines[-1][:-2] + line
                else:
                    lines.append(line)
            for line in lines:
                pipe = pypes.Pype()
                pipe.ExitOnError = 0
                pipe.Arguments = line.split()
                pipe.ParseArguments()
                pipe.Execute()
        return 0

    if '--help' in sys.argv:
        print('Usage: \tvmtk [--ui pad|console]\t\tStart in interactive mode'
              '\n\tvmtk [PYPE]\t\t\tExecute the pype [PYPE]'
              '\n\tvmtk --file [FILE]\t\tExecute the content of file [FILE]')
        return 0

    ui = 'pad'
    if '--ui' in sys.argv and sys.argv.index('--ui') != len(sys.argv) - 1:
        ui = sys.argv[sys.argv.index('--ui') + 1]

    if ui == 'pad':
        try:
            from vmtk import pypepad
        except ImportError:
            print("error")
            ui = 'console'
        else:
            pypepad.RunPypeTkPad()

    if ui == 'console':
        try:
            import readline
        except ImportError:
            pass
        else:
            readline.parse_and_bind("tab: complete")

        while 1:
            try:
                inputString = input("vmtk> ")
            except EOFError:
                sys.stdout.write('\n')
                return 0
            if not inputString:
                continue
            print("Executing", inputString)
            pipe = pypes.Pype()
            pipe.ExitOnError = 0
            pipe.Arguments = inputString.split()
            pipe.ParseArguments()
            try:
                pipe.Execute()
            except Exception:
                continue

    return 0


if __name__ == '__main__':
    sys.exit(main())



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtksurfacekiteremoval as kiteremoval


def test_default_params(aorta_surface, compare_surfaces):
    name = __name__ + '_test_default_params.vtp'
    remover = kiteremoval.vmtkSurfaceKiteRemoval()
    remover.Surface = aorta_surface
    remover.Execute()

    assert compare_surfaces(remover.Surface, name) == True


def test_change_size_factor(aorta_surface, compare_surfaces):
    name = __name__ + '_test_change_size_factor.vtp'
    remover = kiteremoval.vmtkSurfaceKiteRemoval()
    remover.Surface = aorta_surface
    remover.SizeFactor = 0.15
    remover.Execute()

    assert compare_surfaces(remover.Surface, name) == True



#!/usr/bin/env python

from __future__ import print_function, absolute_import # NEED TO STAY AS TOP IMPORT
import sys
import os


if __name__ == '__main__':

    # hack for code compatibility in python 2 and 3 as there is no __future__ module which remaps the new
    # python 3 style input to the equivalent raw_input function in python 2
    try:
        input = raw_input
    except NameError:
        pass

    if sys.platform == 'win32':

        vmtkhome = os.path.dirname(os.path.abspath(__file__))

        if vmtkhome.endswith('bin'):
            vmtkhome = os.path.join(os.path.dirname(os.path.abspath(__file__)),"..")
            os.environ["PYTHONPATH"] = os.path.join(vmtkhome)
        else:
            vmtkhome = os.path.join(os.path.dirname(os.path.abspath(__file__)),"..","..","..")
            os.environ["PYTHONPATH"] = os.path.join(vmtkhome,"lib","${VMTK_PYTHON_VERSION}","site-packages")

        sys.path.append(os.path.join(vmtkhome,"bin"))
        sys.path.append(os.environ["PYTHONPATH"])
        os.environ["PATH"] += os.path.pathsep + os.path.join(vmtkhome,"bin")

    import vtk
    from vmtk import pypes

    vmtkOptions = ['--help', '--ui', '--file']

    if len(sys.argv) > 1 and sys.argv[1] not in vmtkOptions:
        arguments = sys.argv[1:]
        print("Executing", ' '.join(arguments))
        pipe = pypes.Pype()
        pipe.ExitOnError = 0
        pipe.Arguments = arguments
        pipe.ParseArguments()
        pipe.Execute()
        sys.exit(0)
    elif len(sys.argv) > 1 and '--file' in sys.argv:
        fileindex = sys.argv.index('--file')+1
        if fileindex < len(sys.argv):
            inputfile = open(sys.argv[fileindex],'r')
            physicalLines = [line for line in inputfile.readlines() if line and line.strip() and line.strip()[0]!='#']
            lines = []
            for line in physicalLines:
                if lines and lines[-1].endswith('\\\n'):
                    lines[-1] = lines[-1][:-2] + line
                else:
                    lines.append(line)
            for line in lines:
                pipe = pypes.Pype()
                pipe.ExitOnError = 0
                pipe.Arguments = line.split()
                pipe.ParseArguments()
                pipe.Execute()
    elif '--help' in sys.argv:
        print('Usage: \tvmtk [--ui pad|console]\t\tStart in interactive mode\n\tvmtk [PYPE]\t\t\tExecute the pype [PYPE]\n\tvmtk --file [FILE]\t\tExecute the content of file [FILE]')
        sys.exit(0)
    else:
        ui = 'pad'
        if '--ui' in sys.argv and sys.argv.index('--ui') != len(sys.argv)-1:
            ui = sys.argv[sys.argv.index('--ui')+1]

        if ui == 'pad':
            try:
                from vmtk import pypepad
            except ImportError:
                print("error")
                ui = 'console'
            else:
                pypepad.RunPypeTkPad()

        if ui == 'console':
            try:
                import readline
            except ImportError:
                pass
            else:
                readline.parse_and_bind("tab: complete")

            while 1:
                try:
                    inputString = input("vmtk> ")
                except EOFError:
                    sys.stdout.write('\n')
                    sys.exit(0)
                if not inputString:
                    continue
                print("Executing", inputString)
                splitString = inputString.split()
                pipe = pypes.Pype()
                pipe.ExitOnError = 0
                pipe.Arguments = inputString.split()
                pipe.ParseArguments()
                try:
                    pipe.Execute()
                except Exception:
                    continue



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshprojection.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.1 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkMeshProjection(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.ReferenceMesh = None
        self.Mesh = None
        self.Tolerance = 1E-8

        self.SetScriptName('vmtkmeshprojection')
        self.SetScriptDoc('interpolates the point data of a reference mesh onto the input mesh')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['ReferenceMesh','r','vtkUnstructuredGrid',1,'','the reference mesh','vmtkmeshreader'],
            ['Tolerance','tolerance','double',1,'(0.0,)','locator tolerance']
            ])
        self.SetOutputMembers([
            ['Mesh','o','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter']
            ])

    def Execute(self):

        if self.Mesh == None:
            self.PrintError('Error: No Mesh.')

        if self.ReferenceMesh == None:
            self.PrintError('Error: No ReferenceMesh.')

        self.PrintLog('Computing projection.')
        meshProjection = vtkvmtk.vtkvmtkMeshProjection()
        meshProjection.SetInputData(self.Mesh)
        meshProjection.SetReferenceMesh(self.ReferenceMesh)
        meshProjection.SetTolerance(self.Tolerance)
        meshProjection.Update()
        self.Mesh = meshProjection.GetOutput()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmarchingcubes.py,v $
## Language:  Python
## Date:      $Date: 2006/02/06 09:49:42 $
## Version:   $Revision: 1.7 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkMarchingCubes(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None
        self.Surface = None
        self.ArrayName = ''
        self.Level = 0.0
        self.Connectivity = 0

        self.SetScriptName('vmtkmarchingcubes')
        self.SetScriptDoc('generate an isosurface of given level from a 3D image')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['ArrayName','array','str',1,'','name of the array to work with'],
            ['Level','l','float',1,'','graylevel to generate the isosurface at'],
            ['Connectivity','connectivity','bool',1,'','only output the largest connected region of the isosurface']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Image == None:
            self.PrintError('Error: No Image.')

        extent = self.Image.GetExtent()
        translateExtent = vtk.vtkImageTranslateExtent()
        translateExtent.SetInputData(self.Image)
        translateExtent.SetTranslation(-extent[0],-extent[2],-extent[4])
        translateExtent.Update()

        if (self.ArrayName != ''):
            translateExtent.GetOutput().GetPointData().SetActiveScalars(self.ArrayName)

        marchingCubes = vtk.vtkMarchingCubes()
        marchingCubes.SetInputConnection(translateExtent.GetOutputPort())
        marchingCubes.SetValue(0,self.Level)
        marchingCubes.Update()

        self.Surface = marchingCubes.GetOutput()

        if self.Connectivity == 1:
            connectivityFilter = vtk.vtkPolyDataConnectivityFilter()
            connectivityFilter.SetInputData(self.Surface)
            connectivityFilter.SetExtractionModeToLargestRegion()
            connectivityFilter.Update()
            self.Surface = connectivityFilter.GetOutput()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkpointtransform.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.4 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkPointTransform(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Points = []

        self.MatrixCoefficients = []
        self.InvertMatrix = 0

        self.SetScriptName('vmtkpointtransform')
        self.SetScriptDoc('transform points with a provided matrix')
        self.SetInputMembers([
            ['Points','i','float',-1,'','point coordinates'],
            ['MatrixCoefficients','matrix','float',16,'','coefficients of transform matrix'],
            ['InvertMatrix','invert','bool',1,'','invert matrix before applying transformation']
            ])
        self.SetOutputMembers([
            ['Points','o','float',-1,'','output point coordinates']
            ])

    def Execute(self):

        if len(self.Points) % 3 != 0:
            self.PrintError('Error: Points not made up of triplets.')
            return

        matrix = vtk.vtkMatrix4x4()
        matrix.DeepCopy(self.MatrixCoefficients)
        if self.InvertMatrix:
            matrix.Invert()

        transform = vtk.vtkMatrixToLinearTransform()
        transform.SetInput(matrix)

        outputPoints = []
        for i in range(len(self.Points)//3):
            point = [self.Points[3*i+0],self.Points[3*i+1],self.Points[3*i+2]]
            outputPoint = transform.TransformPoint(point)
            outputPoints.append(outputPoint)

        self.Points = outputPoints


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagevesselenhancement.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes
from vmtk import vtkvmtk


class vmtkImageVesselEnhancement(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None
        self.Method = "frangi"

        self.EnhancedImage = None

        self.SigmaMin = 1.0
        self.SigmaMax = 1.0
        self.NumberOfSigmaSteps = 1
        self.SigmaStepMethod = 'equispaced'

        self.ScaledVesselness = 0

        self.Alpha = 0.5
        self.Beta = 0.5
        self.Gamma = 5.0
        self.Alpha1 = 0.5
        self.Alpha2 = 2.0
        self.C = 1E-6
        self.TimeStep = 1E-2
        self.Epsilon = 1E-2
        self.WStrength = 25.0
        self.Sensitivity = 5.0
        self.NumberOfIterations = 0
        self.NumberOfDiffusionSubIterations = 0
        self.BrightObject = True

        self.SetScriptName('vmtkimagevesselenhancement')
        self.SetScriptDoc('compute a feature image for use in segmentation')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['Method','method','str',1,'["frangi","sato","ved","vedm"]'],
            ['SigmaMin','sigmamin','float',1,'(0.0,)'],
            ['SigmaMax','sigmamax','float',1,'(0.0,)'],
            ['NumberOfSigmaSteps','sigmasteps','int',1,'(0,)'],
            ['SigmaStepMethod','stepmethod','str',1,'["equispaced","logarithmic"]'],
            ['ScaledVesselness','scaled','bool',1,'','(frangi)'],
            ['BrightObject','brightobject','bool',1,'','(frangi)'],
            ['Alpha1','alpha1','float',1,'(0.0,)','(sato)'],
            ['Alpha2','alpha2','float',1,'(0.0,)','(sato)'],
            ['Alpha','alpha','float',1,'(0.0,)','(frangi, ved, vedm)'],
            ['Beta','beta','float',1,'(0.0,)','(frangi, ved, vedm)'],
            ['Gamma','gamma','float',1,'(0.0,)','(frangi, ved, vedm)'],
            ['C','c','float',1,'(0.0,)','(ved)'],
            ['TimeStep','timestep','float',1,'(0.0,)','(ved, vedm)'],
            ['Epsilon','epsilon','float',1,'(0.0,)','(ved, vedm)'],
            ['WStrength','wstrength','float',1,'(0.0,)','(ved, vedm)'],
            ['Sensitivity','sensitivity','float',1,'(0.0,)','(ved, vedm)'],
            ['NumberOfIterations','iterations','int',1,'(0,)','(ved, vedm)'],
            ['NumberOfDiffusionSubIterations','subiterations','int',1,'(1,)','(ved, vedm)']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter']
            ])

    def ApplyFrangiVesselness(self):

        vesselness = vtkvmtk.vtkvmtkVesselnessMeasureImageFilter()
        vesselness.SetInputData(self.Image)
        vesselness.SetSigmaMin(self.SigmaMin)
        vesselness.SetSigmaMax(self.SigmaMax)
        vesselness.SetNumberOfSigmaSteps(self.NumberOfSigmaSteps)
        vesselness.SetUseScaledVesselness(self.ScaledVesselness)
        vesselness.SetAlpha(self.Alpha)
        vesselness.SetBeta(self.Beta)
        vesselness.SetGamma(self.Gamma)
        vesselness.SetBrightObject(self.BrightObject)
        if self.SigmaStepMethod == 'equispaced':
            vesselness.SetSigmaStepMethodToEquispaced()
        elif self.SigmaStepMethod == 'logarithmic':
            vesselness.SetSigmaStepMethodToLogarithmic()
        vesselness.Update()

        self.EnhancedImage = vtk.vtkImageData()
        self.EnhancedImage.DeepCopy(vesselness.GetOutput())

    def ApplySatoVesselness(self):

        vesselness = vtkvmtk.vtkvmtkSatoVesselnessMeasureImageFilter()
        vesselness.SetInputData(self.Image)
        vesselness.SetSigmaMin(self.SigmaMin)
        vesselness.SetSigmaMax(self.SigmaMax)
        vesselness.SetNumberOfSigmaSteps(self.NumberOfSigmaSteps)
        vesselness.SetAlpha1(self.Alpha1)
        vesselness.SetAlpha2(self.Alpha2)
        if self.SigmaStepMethod == 'equispaced':
            vesselness.SetSigmaStepMethodToEquispaced()
        elif self.SigmaStepMethod == 'logarithmic':
            vesselness.SetSigmaStepMethodToLogarithmic()
        vesselness.Update()

        self.EnhancedImage = vtk.vtkImageData()
        self.EnhancedImage.DeepCopy(vesselness.GetOutput())

    def ApplyVED(self):

        vesselness = vtkvmtk.vtkvmtkVesselEnhancingDiffusionImageFilter()
        vesselness.SetInputData(self.Image)
        vesselness.SetSigmaMin(self.SigmaMin)
        vesselness.SetSigmaMax(self.SigmaMax)
        vesselness.SetNumberOfSigmaSteps(self.NumberOfSigmaSteps)
        vesselness.SetAlpha(self.Alpha)
        vesselness.SetBeta(self.Beta)
        vesselness.SetGamma(self.Gamma)
        vesselness.SetC(self.C)
        vesselness.SetTimeStep(self.TimeStep)
        vesselness.SetEpsilon(self.Epsilon)
        vesselness.SetWStrength(self.WStrength)
        vesselness.SetSensitivity(self.Sensitivity)
        vesselness.SetNumberOfIterations(self.NumberOfIterations)
        vesselness.SetNumberOfDiffusionSubIterations(self.NumberOfDiffusionSubIterations)
        if self.SigmaStepMethod == 'equispaced':
            vesselness.SetSigmaStepMethodToEquispaced()
        elif self.SigmaStepMethod == 'logarithmic':
            vesselness.SetSigmaStepMethodToLogarithmic()
        vesselness.Update()

        self.EnhancedImage = vtk.vtkImageData()
        self.EnhancedImage.DeepCopy(vesselness.GetOutput())

    def ApplyVEDManniesing(self):

        vesselness = vtkvmtk.vtkvmtkVesselEnhancingDiffusion3DImageFilter()
        vesselness.SetInputData(self.Image)
        vesselness.SetSigmaMin(self.SigmaMin)
        vesselness.SetSigmaMax(self.SigmaMax)
        vesselness.SetNumberOfSigmaSteps(self.NumberOfSigmaSteps)
        vesselness.SetAlpha(self.Alpha)
        vesselness.SetBeta(self.Beta)
        vesselness.SetGamma(self.Gamma)
        vesselness.SetTimeStep(self.TimeStep)
        vesselness.SetEpsilon(self.Epsilon)
        vesselness.SetOmega(self.WStrength)
        vesselness.SetSensitivity(self.Sensitivity)
        vesselness.SetNumberOfIterations(self.NumberOfIterations)
        vesselness.SetRecalculateVesselness(self.NumberOfDiffusionSubIterations)
        if self.SigmaStepMethod == 'equispaced':
            vesselness.SetSigmaStepMethodToEquispaced()
        elif self.SigmaStepMethod == 'logarithmic':
            vesselness.SetSigmaStepMethodToLogarithmic()
        vesselness.Update()

        self.EnhancedImage = vtk.vtkImageData()
        self.EnhancedImage.DeepCopy(vesselness.GetOutput())

    def Execute(self):

        if self.Image == None:
            self.PrintError('Error: No input image.')

        if self.SigmaMax < self.SigmaMin:
            self.SigmaMax = self.SigmaMin

        if( self.Image.GetScalarType() != vtk.VTK_FLOAT):
          # filters only work on float
          print("input type not of type float, casting to float")
          #TODO use rescale filter for proper mapping
          cast = vtk.vtkImageCast()
          cast.SetInputData(self.Image)
          cast.SetOutputScalarTypeToFloat()
          cast.Update()
          self.Image.DeepCopy(cast.GetOutput())

        if(self.Image.GetDataDimension() != 3):
          self.PrintError('Error: unsupported image dimension, expected {0}D image'.format(3))

        if self.Method == 'frangi':
            self.ApplyFrangiVesselness()
        elif self.Method == 'sato':
            self.ApplySatoVesselness()
        elif self.Method == 'ved':
            self.ApplyVED()
        elif self.Method == 'vedm':
            self.ApplyVEDManniesing()
        else:
            self.PrintError('Error: unsupported vessel enhancement method')

        self.Image = self.EnhancedImage


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkmarchingcubes as marchingcubes


@pytest.fixture()
def level_set_image(input_datadir):
    import vmtk.vmtkimagereader as reader
    import os
    read = reader.vmtkImageReader()
    read.InputFileName = os.path.join(input_datadir, 'aorta-final-levelset.mha')
    read.Execute()

    return read.Image


def test_marching_cubes_default(level_set_image, compare_surfaces):
    name = __name__ + '_test_marching_cubes_default.vtp'
    mc = marchingcubes.vmtkMarchingCubes()
    mc.Image = level_set_image
    mc.Level = 0.0
    mc.Execute()

    assert compare_surfaces(mc.Surface, name) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshdatareader.py,v $
## Language:  Python
## Date:      $Date: 2006/07/27 08:27:40 $
## Version:   $Revision: 1.13 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import os
import gzip
import vtk
from vmtk import vtkvmtk

from vmtk import pypes


class vmtkMeshDataReader(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.TetrInFileName = ''
        self.Mesh = 0

        self.DataFileDirectory = '.'
        self.DataFilePrefix = ''
        self.DataFileType = 'tetr.vel'

        self.DataFileName = ''

        self.ArrayNamePrefix = ''

        self.Compressed = 1
        self.UnNormalize = 1

        self.MinVariableId = -1
        self.MaxVariableId = -1

        self.Radius = 1.0
        self.Viscosity = 1.0
        self.Density = 1.0
        self.ReD = 0.0
        self.Alpha = 0.0

        self.SetScriptName('vmtkmeshdatareader')
        self.SetScriptDoc('read data associated with a mesh')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['DataFileDirectory','directory','str',1,'','directory where the data files reside'],
            ['DataFilePrefix','prefix','str',1,'','data file name prefix (e.g. foo_)'],
            ['DataFileName','datafile','str',1,'','data file name (e.g. foo.dat) excluding directory - overrides prefix'],
            ['DataFileType','filetype','str',1,'','data file name type (tetr.out, tetr.vel, tetr.wss, tetr.pres, tetr.ini, pointdata)'],
            ['ArrayNamePrefix','arrayprefix','str',1,'','prefix to prepend to array names in output'],
            ['TetrInFileName','tetrinfile','str',1,'','name of the tetr.in file'],
            ['Compressed','compressed','bool',1,'','toggle reading gzip compressed file'],
            ['UnNormalize','unnormalize','bool',1,'','unnormalize quantities - tetr.* only'],
            ['MinVariableId','minvarid','int',1,'(0,)','read variables starting from the nth column - pointdata only'],
            ['MaxVariableId','maxvarid','int',1,'(0,)','read variables up to the nth column - pointdata only']
            ])
        self.SetOutputMembers([
            ['Mesh','o','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter']
            ])

    def ReadTetrInDimensionalParameters(self):

        tetrinf=open(self.TetrInFileName, 'r')
        tetrinline = ''
        while tetrinline.strip() != '$radius':
            tetrinline = tetrinf.readline()
            if tetrinline == '':
                break
        if tetrinline.strip() == '$radius':
            tetrinline = tetrinf.readline()
            self.Radius = float(tetrinline.strip())

        tetrinf.seek(0)
        tetrinline = ''
        while tetrinline.strip() != '$viscosity':
            tetrinline = tetrinf.readline()
            if tetrinline == '':
                break
        if tetrinline.strip() == '$viscosity':
            tetrinline = tetrinf.readline()
            self.Viscosity = float(tetrinline.strip())

        tetrinf.seek(0)
        tetrinline = ''
        while tetrinline.strip() != '$density':
            tetrinline = tetrinf.readline()
            if tetrinline == '':
                break
        if tetrinline.strip() == '$density':
            tetrinline = tetrinf.readline()
            self.Density = float(tetrinline.strip())

        tetrinf.seek(0)
        tetrinline = ''
        while tetrinline.strip() != '$red':
            tetrinline = tetrinf.readline()
            if tetrinline == '':
                break
        if tetrinline.strip() == '$red':
            tetrinline = tetrinf.readline()
            self.ReD = float(tetrinline.strip())
        else:
            tetrinf.seek(0)
            while tetrinline.strip() != '$rer':
                tetrinline = tetrinf.readline()
                if tetrinline == '':
                    break
            if tetrinline.strip() == '$rer':
                tetrinline = tetrinf.readline()
                self.ReD = float(tetrinline.strip())*2.0

        tetrinf.seek(0)
        tetrinline = ''
        while tetrinline.strip() != '$alpha':
            tetrinline = tetrinf.readline()
            if tetrinline == '':
                break
        if tetrinline.strip() == '$alpha':
            tetrinline = tetrinf.readline()
            self.Alpha = float(tetrinline.strip())

    def ReadTetrOutFile(self,filename,arrayname):
        self.PrintLog('Reading '+filename+'.')
        if self.UnNormalize == 1:
            self.ReadTetrInDimensionalParameters()
        f = None
        if (self.Compressed == 1):
            f=gzip.open(filename, 'r')
        else:
            f=open(filename, 'r')
        line = f.readline()
        while (line!='$output'):
            line = f.readline()
        line = f.readline()
        splitline = line.split(' ')
        numberOfTuples = int(splitline[0])
        outputArray = vtk.vtkDoubleArray()
        outputArray.SetName(arrayname)
        outputArray.SetNumberOfComponents(3)
        outputArray.SetNumberOfTuples(numberOfTuples)
        velocityUnNormalizationFactor = self.ReD / (2.0 * self.Radius) * self.Viscosity / self.Density
        for i in range(numberOfTuples):
            line = f.readline()
            splitline = line.split(' ')
            value0 = float(splitline[5])
            value1 = float(splitline[6])
            value2 = float(splitline[7])
            if self.UnNormalize ==1:
                value0 *= velocityUnNormalizationFactor
                value1 *= velocityUnNormalizationFactor
                value2 *= velocityUnNormalizationFactor
            outputArray.SetComponent(i,0,value0)
            outputArray.SetComponent(i,1,value1)
            outputArray.SetComponent(i,2,value2)
        self.Mesh.GetPointData().AddArray(outputArray)

    def ReadTetrVelFile(self,filename,arrayname):
        self.PrintLog('Reading '+filename+'.')
        if self.UnNormalize == 1:
            self.ReadTetrInDimensionalParameters()
        if (self.Compressed == 1):
            f=gzip.open(filename, 'r')
        else:
            f=open(filename, 'r')
        lines = f.readlines()
        line = lines[0]
        lineoffset = 1
        splitline = line.split(' ')
        numberOfTuples = int(splitline[0])
        iteration = float(splitline[1])
        outputArray = vtk.vtkDoubleArray()
        outputArray.SetName(arrayname)
        outputArray.SetNumberOfComponents(3)
        outputArray.SetNumberOfTuples(numberOfTuples)
        velocityUnNormalizationFactor = self.ReD / (2.0 * self.Radius) * self.Viscosity / self.Density
        for i in range(numberOfTuples):
            line = lines[i+lineoffset]
            splitline = line.split(' ')
            value0 = float(splitline[0])
            value1 = float(splitline[1])
            value2 = float(splitline[2])
            if self.UnNormalize ==1:
                value0 *= velocityUnNormalizationFactor
                value1 *= velocityUnNormalizationFactor
                value2 *= velocityUnNormalizationFactor
            outputArray.SetComponent(i,0,value0)
            outputArray.SetComponent(i,1,value1)
            outputArray.SetComponent(i,2,value2)
        self.Mesh.GetPointData().AddArray(outputArray)

    def ReadTetrWSSFile(self,filename,arrayname):
        self.PrintLog('Reading '+filename+'.')
        if self.UnNormalize == 1:
            self.ReadTetrInDimensionalParameters()
        if (self.Compressed == 1):
            f=gzip.open(filename, 'r')
        else:
            f=open(filename, 'r')
        lines = f.readlines()
        line = lines[1]
        lineoffset = 2
        numberOfTuples = int(line)
        numberOfPoints = self.Mesh.GetNumberOfPoints()
        outputArray = vtk.vtkDoubleArray()
        outputArray.SetName(arrayname)
        outputArray.SetNumberOfComponents(3)
        outputArray.SetNumberOfTuples(numberOfPoints)
        outputArray.FillComponent(0,0.0)
        outputArray.FillComponent(1,0.0)
        outputArray.FillComponent(2,0.0)
        tetrinf=open(self.TetrInFileName, 'r')
        tetrinline = ''
        while tetrinline.strip() != '$wnode':
            tetrinline = tetrinf.readline()
        tetrinline = tetrinf.readline()
        wssUnNormalizationFactor = 4.0 * self.Viscosity * (self.ReD / (2.0 * self.Radius) * self.Viscosity / self.Density) / self.Radius
        for i in range(numberOfTuples):
            line = lines[i+lineoffset]
            splitline = line.split(' ')
            value0 = float(splitline[0])
            value1 = float(splitline[1])
            value2 = float(splitline[2])
            if self.UnNormalize ==1:
                value0 *= wssUnNormalizationFactor
                value1 *= wssUnNormalizationFactor
                value2 *= wssUnNormalizationFactor
            tetrinline = tetrinf.readline()
            pointId = int(tetrinline.strip().split(' ')[1]) - 1
            outputArray.SetComponent(pointId,0,value0)
            outputArray.SetComponent(pointId,1,value1)
            outputArray.SetComponent(pointId,2,value2)
        self.Mesh.GetPointData().AddArray(outputArray)

    def ReadTetrIniFile(self,filename,arrayname):
        self.PrintLog('Reading '+filename+'.')
        if (self.Compressed == 1):
            f=gzip.open(filename, 'r')
        else:
            f=open(filename, 'r')
        line = f.readline()
        while (line!='$vel_old1'):
            line = f.readline()
        line = f.readline()
        line = f.readline()
        numberOfTuples = int(line)
        outputArray = vtk.vtkDoubleArray()
        outputArray.SetName(arrayname)
        outputArray.SetNumberOfComponents(3)
        outputArray.SetNumberOfTuples(numberOfTuples)
        for i in range(numberOfTuples):
            line = f.readline()
            splitline = line.split(' ')
            value0 = float(splitline[0])
            value1 = float(splitline[1])
            value2 = float(splitline[2])
            outputArray.SetComponent(i,0,value0)
            outputArray.SetComponent(i,1,value1)
            outputArray.SetComponent(i,2,value2)
        self.Mesh.GetPointData().AddArray(outputArray)

    def ReadPointDataFile(self,filename):
        self.PrintLog('Reading '+filename+'.')
        if (self.Compressed == 1):
            f=gzip.open(filename, 'r')
        else:
            f=open(filename, 'r')
        lines = f.readlines()
        line = lines[0]
        splitline = line.strip().split(' ')
        numberOfVariables = len(splitline)
        numberOfPoints = self.Mesh.GetNumberOfPoints()
        arrayNames = []
        for i in range(numberOfVariables):
            arrayNames.append(filename+'_'+splitline[i])
            if (self.MinVariableId != -1) & (i < self.MinVariableId):
                continue
            if (self.MaxVariableId != -1) & (i > self.MaxVariableId):
                continue
            outputArray = vtk.vtkDoubleArray()
            outputArray.SetName(arrayNames[i])
            outputArray.SetNumberOfComponents(1)
            outputArray.SetNumberOfTuples(numberOfPoints)
            self.Mesh.GetPointData().AddArray(outputArray)
        pointId = 0
        for i in range(1,len(lines)):
            line = lines[i]
            splitline = line.strip().split(' ')
            if len(splitline) != numberOfVariables:
                self.PrintError('Error: pointdata file corrupted.');
            for j in range(numberOfVariables):
                if (self.MinVariableId != -1) & (i < self.MinVariableId):
                    continue
                if (self.MaxVariableId != -1) & (i > self.MaxVariableId):
                    continue
                self.Mesh.GetPointData().GetArray(arrayNames[j]).SetComponent(pointId,0,float(splitline[j]))
            pointId += 1

    def Execute(self):

        if self.Mesh == None:
            self.PrintError('Error: No input mesh.')

        dataFilenames = []

        if not self.DataFileName:
            if self.DataFilePrefix == '':
                self.DataFilePrefix = self.DataFileType
            filenames = os.listdir(self.DataFileDirectory)
            for filename in filenames:
                if filename.find(self.DataFilePrefix) != -1:
                    dataFilenames.append(filename)
            dataFilenames.sort()
        else:
            dataFilenames.append(self.DataFileName)

        for name in dataFilenames:
            filename = self.DataFileDirectory + '/' + name
            arrayname = self.ArrayNamePrefix + name
            if (self.DataFileType == 'tetr.out'):
                self.ReadTetrOutFile(filename,arrayname)
            elif (self.DataFileType == 'tetr.vel'):
                self.ReadTetrVelFile(filename,arrayname)
            elif (self.DataFileType == 'tetr.wss'):
                self.ReadTetrWSSFile(filename,arrayname)
            elif (self.DataFileType == 'tetr.ini'):
                self.ReadTetrIniFile(filename,arrayname)
            elif (self.DataFileType == 'pointdata'):
                if self.UnNormalize == 1:
                    self.PrintError('Error: UnNormalize not supported for pointdata DataFileType.')
                self.ReadPointDataFile(filename)
            else:
                self.PrintError('Unsupported DataFileType.')


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtksurfacesubdivision as surfacesubdivision


@pytest.mark.parametrize("method,numsubdivisions,expected_points,expected_cells,paramid", [
    ('linear', 1, 25800, 51600, '0'),
    ('butterfly', 1, 25800, 51600, '1'),
    ('loop', 1, 25800, 51600, '2'),
    ('linear', 2, 103000, 206000, '3'),
    ('butterfly', 2, 103000, 206000, '4'),
    ('loop', 2, 103000, 206000, '5'),
])
def test_subdivision_methods_and_iterations(aorta_surface, compare_surfaces,
                                            method, numsubdivisions, paramid,
                                            expected_points, expected_cells):
    name = __name__ + '_test_subdivision_methods_and_iterations_' + paramid + '.vtp'
    subdivider = surfacesubdivision.vmtkSurfaceSubdivision()
    subdivider.Surface = aorta_surface
    subdivider.Method = method
    subdivider.NumberOfSubdivisions = numsubdivisions
    subdivider.Execute()

    assert compare_surfaces(subdivider.Surface, name) == True
    assert subdivider.Surface.GetNumberOfPoints() >= expected_points
    assert subdivider.Surface.GetNumberOfCells() >= expected_cells



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfaceregiondrawing.py,v $
## Language:  Python
## Date:      $Date: 2006/05/26 12:35:13 $
## Version:   $Revision: 1.9 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vmtkrenderer
from vmtk import pypes


class vmtkSurfaceRegionDrawing(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.Actor = None
        self.ContourWidget = None
        self.Interpolator = None
        self.Binary = 1
        self.OutsideValue = 1.0
        self.InsideValue = 0.0

        self.ContourScalarsArrayName = 'ContourScalars'

        self.SetScriptName('vmtksurfaceregiondrawing')
        self.SetScriptDoc('draw a closed contour on a surface and generate a distance field on the surface')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['Binary','binary','bool',1,'','fill contour with inside value instead of distance to contour'],
            ['InsideValue','inside','float',1,'','value with which the surface within the contour is filled'],
            ['OutsideValue','outside','float',1,'','value with which the surface outside the contour is filled'],
            ['ContourScalarsArrayName','array','str',1,'','the name of the array where the generated scalars are stored'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def ScalarsCallback(self, obj):

        rep = vtk.vtkOrientedGlyphContourRepresentation.SafeDownCast(self.ContourWidget.GetRepresentation())

        pointIds = vtk.vtkIdList()
        self.Interpolator.GetContourPointIds(rep,pointIds)

        points = vtk.vtkPoints()
        points.SetNumberOfPoints(pointIds.GetNumberOfIds())

        for i in range(pointIds.GetNumberOfIds()):
            pointId = pointIds.GetId(i)
            point = self.Surface.GetPoint(pointId)
            points.SetPoint(i,point)

        selectionFilter = vtk.vtkSelectPolyData()
        selectionFilter.SetInputData(self.Surface)
        selectionFilter.SetLoop(points)
        selectionFilter.GenerateSelectionScalarsOn()
        selectionFilter.SetSelectionModeToSmallestRegion()
        selectionFilter.Update()

        selectionScalars = selectionFilter.GetOutput().GetPointData().GetScalars()

        contourScalars = self.Surface.GetPointData().GetArray(self.ContourScalarsArrayName)

        for i in range(contourScalars.GetNumberOfTuples()):
            selectionValue = selectionScalars.GetTuple1(i)
            if self.Binary:
                if selectionValue < 0.0:
                    contourScalars.SetTuple1(i,self.InsideValue)
            else:
                contourValue = contourScalars.GetTuple1(i)
                if (not contourValue < 0.0 and selectionValue < 0.0) or (contourValue < 0.0 and selectionValue < contourValue):
                    contourScalars.SetTuple1(i,selectionValue)

        self.Actor.GetMapper().SetScalarRange(contourScalars.GetRange(0))

        self.Surface.Modified()

        self.ContourWidget.Initialize()

    def DeleteContourCallback(self, obj):
        self.ContourWidget.Initialize()

    def InteractCallback(self, obj):
        if self.ContourWidget.GetEnabled() == 1:
            self.ContourWidget.SetEnabled(0)
        else:
            self.ContourWidget.SetEnabled(1)

    def Display(self):

        self.vmtkRenderer.Render()

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: no Surface.')

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self)

        triangleFilter = vtk.vtkTriangleFilter()
        triangleFilter.SetInputData(self.Surface)
        triangleFilter.Update()

        self.Surface = triangleFilter.GetOutput()

        contourScalars = vtk.vtkDoubleArray()
        contourScalars.SetNumberOfComponents(1)
        contourScalars.SetNumberOfTuples(self.Surface.GetNumberOfPoints())
        contourScalars.SetName(self.ContourScalarsArrayName)
        contourScalars.FillComponent(0,self.OutsideValue)

        self.Surface.GetPointData().AddArray(contourScalars)
        self.Surface.GetPointData().SetActiveScalars(self.ContourScalarsArrayName)

        mapper = vtk.vtkPolyDataMapper()
        mapper.SetInputData(self.Surface)
        mapper.ScalarVisibilityOn()
        self.Actor = vtk.vtkActor()
        self.Actor.SetMapper(mapper)
        self.Actor.GetMapper().SetScalarRange(-1.0,0.0)
        self.vmtkRenderer.Renderer.AddActor(self.Actor)

        self.ContourWidget = vtk.vtkContourWidget()
        self.ContourWidget.SetInteractor(self.vmtkRenderer.RenderWindowInteractor)

        rep = vtk.vtkOrientedGlyphContourRepresentation.SafeDownCast(self.ContourWidget.GetRepresentation())
        rep.GetLinesProperty().SetColor(1, 0.2, 0)
        rep.GetLinesProperty().SetLineWidth(3.0)

        pointPlacer = vtk.vtkPolygonalSurfacePointPlacer()
        pointPlacer.AddProp(self.Actor)
        pointPlacer.GetPolys().AddItem(self.Surface)
        rep.SetPointPlacer(pointPlacer)

        self.Interpolator = vtk.vtkPolygonalSurfaceContourLineInterpolator()
        self.Interpolator.GetPolys().AddItem(self.Surface)
        rep.SetLineInterpolator(self.Interpolator)

        self.vmtkRenderer.AddKeyBinding('space','Generate scalars',self.ScalarsCallback)
        self.vmtkRenderer.AddKeyBinding('d','Delete contour',self.DeleteContourCallback)
        self.vmtkRenderer.AddKeyBinding('i','Start interaction',self.InteractCallback)
        self.Display()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshreader.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.16 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import vtk
from vmtk import vtkvmtk

from vmtk import pypes


class vmtkMeshReader(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Format = ''
        self.GuessFormat = 1
        self.InputFileName = ''
        self.Mesh = 0
        self.Output = 0

        self.GhostNodes = 1
        self.VolumeElementsOnly = 0

        self.CellEntityIdsArrayName = 'CellEntityIds'

        self.SetScriptName('vmtkmeshreader')
        self.SetScriptDoc('read a mesh and stores it in a vtkUnstructuredGrid object')
        self.SetInputMembers([
            ['Format','f','str',1,'["vtkxml", "vtkxmlp", "vtk","fdneut","ngneut","tecplot","tetgen","gambit"]','file format (fdneut - FIDAP neutral format, ngneut - Netgen neutral format)'],
            ['GuessFormat','guessformat','bool',1,'','guess file format from extension'],
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh'],
            ['InputFileName','ifile','str',1,'','input file name'],
            ['GhostNodes','ghostnodes','bool',1,'','store all nodes for 9-noded quads, 7-noded triangles, 27-noded hexahedra, 18-noded wedges; otherwise, store them as 8-noded quads, 6-noded triangles, 20-noded hexahedra, 15-noded wedges - fdneut only'],
            ['VolumeElementsOnly','volumeelementsonly','bool',1,'','only read volume elements - fdneut and ngneut'],
            ['CellEntityIdsArrayName','entityidsarray','str',1,'','name of the array where entity ids have to be stored - ngneut and tetgen']
            ])
        self.SetOutputMembers([
            ['Mesh','o','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter'],
            ['CellEntityIdsArrayName','entityidsarray','str',1,'','name of the array where entity ids have been stored - ngneut and tetgen']
            ])

    def ReadTetGenMeshFile(self):
        if (self.InputFileName == ''):
            self.PrintError('Error: no InputFileName.')
        self.PrintLog('Reading TetGen mesh file.')
        import os.path
        inputFileName = os.path.splitext(self.InputFileName)[0]
        reader = vtkvmtk.vtkvmtkTetGenReader()
        reader.SetFileName(inputFileName)
        reader.SetBoundaryDataArrayName(self.CellEntityIdsArrayName)
        reader.Update()
        self.Mesh = reader.GetOutput()

    def ReadGAMBITMeshFile(self):
        if (self.InputFileName == ''):
            self.PrintError('Error: no InputFileName.')
        self.PrintLog('Reading GAMBIT mesh file.')
        reader = vtk.vtkGAMBITReader()
        reader.SetFileName(self.InputFileName)
        reader.Update()
        self.Mesh = reader.GetOutput()

    def ReadVTKMeshFile(self):
        if (self.InputFileName == ''):
            self.PrintError('Error: no InputFileName.')
        self.PrintLog('Reading VTK mesh file.')
        reader = vtk.vtkUnstructuredGridReader()
        reader.SetFileName(self.InputFileName)
        reader.Update()
        self.Mesh = reader.GetOutput()

    def ReadVTKXMLMeshFile(self):
        if (self.InputFileName == ''):
            self.PrintError('Error: no InputFileName.')
        self.PrintLog('Reading VTK XML mesh file.')
        reader = vtk.vtkXMLUnstructuredGridReader()
        reader.SetFileName(self.InputFileName)
        reader.Update()
        self.Mesh = reader.GetOutput()

    def ReadVTKXMLPMeshFile(self):
        if (self.InputFileName == ''):
            self.PrintError('Error: no InputFileName.')
        self.PrintLog('Reading VTK XML Parallel format mesh file.')
        reader = vtk.vtkXMLPUnstructuredGridReader()
        reader.SetFileName(self.InputFileName)
        reader.Update()
        self.Mesh = reader.GetOutput()

    def ReadXdaMeshFile(self):
        if (self.InputFileName == ''):
            self.PrintError('Error: no InputFileName.')
        self.PrintError('Error: Xda reader not yet implemented.')
        return
#        self.PrintLog('Reading Xda mesh file.')
#        reader = vtkvmtk.vtkvmtkXdaReader()
#        reader.SetFileName(self.InputFileName)
#        reader.Update()
#        self.Mesh = reader.GetOutput()

    def ReadFDNEUTMeshFile(self):
        if (self.InputFileName == ''):
            self.PrintError('Error: no InputFileName.')
        self.PrintLog('Reading FDNEUT mesh file.')
        reader = vtkvmtk.vtkvmtkFDNEUTReader()
        reader.SetFileName(self.InputFileName)
        reader.SetGhostNodes(self.GhostNodes)
        reader.SetSingleCellDataEntityArrayName(self.CellEntityIdsArrayName)
        reader.SetVolumeElementsOnly(self.VolumeElementsOnly)
        reader.Update()
        self.Mesh = reader.GetOutput()

    def ReadNGNEUTMeshFile(self):
        if (self.InputFileName == ''):
            self.PrintError('Error: no InputFileName.')
        self.PrintLog('Reading ngneut mesh file.')
        f=open(self.InputFileName, 'r')
        self.Mesh = vtk.vtkUnstructuredGrid()
        self.MeshPoints = vtk.vtkPoints()
        line = f.readline()
        numberOfPoints = int(line)
        self.MeshPoints.SetNumberOfPoints(numberOfPoints)
        for i in range(numberOfPoints):
            line = f.readline()
            splitLine = line.strip().split()
            point = [float(splitLine[0]),float(splitLine[1]),float(splitLine[2])]
            self.MeshPoints.SetPoint(i,point)
        self.Mesh.SetPoints(self.MeshPoints)
        self.Mesh.Allocate(numberOfPoints*4,1000)
        line = f.readline()
        numberOfVolumeCells = int(line)
        cellEntityIdArray = vtk.vtkIntArray()
        cellEntityIdArray.SetName(self.CellEntityIdsArrayName)
        for i in range(numberOfVolumeCells):
            line = f.readline()
            splitLine = line.strip().split()
            cellType = -1
            numberOfCellPoints = len(splitLine)-1
            pointIds = vtk.vtkIdList()
            if numberOfCellPoints == 4:
                cellType = 10
#                pointIds.InsertNextId(int(splitLine[0+1])-1)
#                pointIds.InsertNextId(int(splitLine[1+1])-1)
#                pointIds.InsertNextId(int(splitLine[2+1])-1)
#                pointIds.InsertNextId(int(splitLine[3+1])-1)
                pointIds.InsertNextId(int(splitLine[1+1])-1)
                pointIds.InsertNextId(int(splitLine[2+1])-1)
                pointIds.InsertNextId(int(splitLine[3+1])-1)
                pointIds.InsertNextId(int(splitLine[0+1])-1)
            elif numberOfCellPoints == 10:
                cellType = 24
#                pointIds.InsertNextId(int(splitLine[0+1])-1)
#                pointIds.InsertNextId(int(splitLine[1+1])-1)
#                pointIds.InsertNextId(int(splitLine[2+1])-1)
#                pointIds.InsertNextId(int(splitLine[3+1])-1)
#                pointIds.InsertNextId(int(splitLine[4+1])-1)
#                pointIds.InsertNextId(int(splitLine[7+1])-1)
#                pointIds.InsertNextId(int(splitLine[5+1])-1)
#                pointIds.InsertNextId(int(splitLine[6+1])-1)
#                pointIds.InsertNextId(int(splitLine[8+1])-1)
#                pointIds.InsertNextId(int(splitLine[9+1])-1)
                pointIds.InsertNextId(int(splitLine[1+1])-1)
                pointIds.InsertNextId(int(splitLine[2+1])-1)
                pointIds.InsertNextId(int(splitLine[3+1])-1)
                pointIds.InsertNextId(int(splitLine[0+1])-1)
                pointIds.InsertNextId(int(splitLine[7+1])-1)
                pointIds.InsertNextId(int(splitLine[9+1])-1)
                pointIds.InsertNextId(int(splitLine[8+1])-1)
                pointIds.InsertNextId(int(splitLine[4+1])-1)
                pointIds.InsertNextId(int(splitLine[5+1])-1)
                pointIds.InsertNextId(int(splitLine[6+1])-1)
            entityId = int(splitLine[0])
            cellEntityIdArray.InsertNextValue(entityId)
            self.Mesh.InsertNextCell(cellType,pointIds)
        if self.VolumeElementsOnly == 0:
            line = f.readline()
            numberOfSurfaceCells = int(line)
            for i in range(numberOfSurfaceCells):
                line = f.readline()
                splitLine = line.strip().split()
                cellType = -1
                numberOfCellPoints = len(splitLine)-1
                pointIds = vtk.vtkIdList()
                if numberOfCellPoints == 3:
                    cellType = 5
                    for j in range(numberOfCellPoints):
                        pointIds.InsertNextId(int(splitLine[j+1])-1)
                elif numberOfCellPoints == 6:
                    cellType = 22
                    for j in range(3):
                        pointIds.InsertNextId(int(splitLine[j+1])-1)
                    pointIds.InsertNextId(int(splitLine[5+1])-1)
                    pointIds.InsertNextId(int(splitLine[3+1])-1)
                    pointIds.InsertNextId(int(splitLine[4+1])-1)
                entityId = int(splitLine[0])
                cellEntityIdArray.InsertNextValue(entityId)
                self.Mesh.InsertNextCell(cellType,pointIds)
        self.Mesh.Squeeze()
        self.Mesh.GetCellData().AddArray(cellEntityIdArray)

    def ReadTecplotMeshFile(self):
        self.PrintLog('Reading Tecplot surface file.')
        if self.InputFileName[-3:] == '.gz':
            import gzip
            f=gzip.open(self.InputFileName, 'r')
        else:
            f=open(self.InputFileName, 'r')
        line = f.readline()
        if line.split()[0] == 'TITLE':
            line = f.readline()
        if (line.split()[0] == 'VARIABLES') | (line.split('=')[0] == 'VARIABLES'):
            arrayNames = line.split('=')[1].strip().split(',')
            arrayNames[0:3] = []
            self.PrintLog("ArrayNames" + str(arrayNames))
            line = f.readline()
        if line.split()[0] == 'ZONE':
            lineNid = line.find('N=')
            lineN = line[lineNid : lineNid+line[lineNid:].find(',') ].split('=')[1]
            numberOfNodes = int(lineN)
            lineEid = line.find('E=')
            lineE = line[lineEid : lineEid+line[lineEid:].find(',') ].split('=')[1]
            numberOfElements = int(lineE)
        self.PrintLog("Reading " + str(numberOfNodes)+" nodes.")
        points = vtk.vtkPoints()
        cells = vtk.vtkCellArray()
        points.SetNumberOfPoints(numberOfNodes)
        self.Mesh = vtk.vtkUnstructuredGrid()
        self.Mesh.SetPoints(points)
        for arrayName in arrayNames:
            array = vtk.vtkDoubleArray()
            array.SetName(arrayName)
            array.SetNumberOfTuples(numberOfNodes)
            self.Mesh.GetPointData().AddArray(array)
        data = f.read().split()
        dataCounter = 0
        for i in range(numberOfNodes):
            point = [float(data[dataCounter]),float(data[dataCounter+1]),float(data[dataCounter+2])]
            dataCounter += 3
            points.SetPoint(i,point)
            for j in range(len(arrayNames)):
                self.Mesh.GetPointData().GetArray(arrayNames[j]).SetComponent(i,0,float(data[dataCounter]))
                dataCounter += 1
        self.PrintLog("Reading " + str(numberOfElements)+" elements.")
        cellIds = vtk.vtkIdList()
        for i in range(numberOfElements):
            cellIds.Initialize()
            cellIds.InsertNextId(int(data[dataCounter])-1)
            dataCounter += 1
            cellIds.InsertNextId(int(data[dataCounter])-1)
            dataCounter += 1
            cellIds.InsertNextId(int(data[dataCounter])-1)
            dataCounter += 1
            cellIds.InsertNextId(int(data[dataCounter])-1)
            dataCounter += 1
            legal = 1
            for j in range(cellIds.GetNumberOfIds()):
                if cellIds.GetId(j) < 0:
                    legal = 0
                    break
            if not legal:
                continue
            cells.InsertNextCell(cellIds)
        self.Mesh.SetCells(10,cells)
        self.Mesh.Update()

    def Execute(self):

        extensionFormats = {'vtu':'vtkxml',
                            'vtkxml':'vtkxml',
                            'pvtu':'vtkxmlp',
                            'vtk':'vtk',
                            'FDNEUT':'fdneut',
                            'xda':'xda',
                            'neu':'ngneut',
                            'gneu':'gambit',
                            'tec':'tecplot',
                            'node':'tetgen',
                            'ele':'tetgen'}

        if self.InputFileName == 'BROWSER':
            import tkinter.filedialog
            import os.path
            initialDir = pypes.pypeScript.lastVisitedPath
            self.InputFileName = tkinter.filedialog.askopenfilename(title="Input mesh",initialdir=initialDir)
            pypes.pypeScript.lastVisitedPath = os.path.dirname(self.InputFileName)
            if not self.InputFileName:
                self.PrintError('Error: no InputFileName.')

        if self.GuessFormat and self.InputFileName and not self.Format:
            import os.path
            extension = os.path.splitext(self.InputFileName)[1]
            if extension:
                extension = extension[1:]
                if extension in list(extensionFormats.keys()):
                    self.Format = extensionFormats[extension]

        if (self.Format == 'vtk'):
            self.ReadVTKMeshFile()
        elif (self.Format == 'vtkxml'):
            self.ReadVTKXMLMeshFile()
        elif (self.Format == 'vtkxmlp'):
            self.ReadVTKXMLPMeshFile()
        elif (self.Format == 'gambit'):
            self.ReadGAMBITMeshFile()
        elif (self.Format == 'fdneut'):
            self.ReadFDNEUTMeshFile()
        elif (self.Format == 'ngneut'):
            self.ReadNGNEUTMeshFile()
        elif (self.Format == 'xda'):
            self.ReadXdaMeshFile()
        elif (self.Format == 'tecplot'):
            self.ReadTecplotMeshFile()
        elif (self.Format == 'tetgen'):
            self.ReadTetGenMeshFile()
        else:
            self.PrintError('Error: unsupported format '+ self.Format + '.')

        self.Output = self.Mesh


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfacekiteremoval.py,v $
## Language:  Python
## Date:      $Date: 2006/03/01 11:53:48 $
## Version:   $Revision: 1.1 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import vtk
from vmtk import vtkvmtk
from vmtk import pypes


class vmtkSurfaceKiteRemoval(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None

        self.SizeFactor = 0.1

        self.SetScriptName('vmtksurfacekiteremoval')
        self.SetScriptDoc('remove small kites in a surface mesh to avoid Taubin smoothing artifacts')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['SizeFactor','sizefactor','float',1,'(0.0,)','ratio between local and global average triangle area']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        kiteRemoval = vtkvmtk.vtkvmtkPolyDataKiteRemovalFilter()
        kiteRemoval.SetInputData(self.Surface)
        kiteRemoval.SetSizeFactor(self.SizeFactor)
        kiteRemoval.Update()

        self.Surface = kiteRemoval.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!${PYTHON_SHEBANG}

## Program:   PypeS
## Module:    $RCSfile: pypebatch.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:47:30 $
## Version:   $Revision: 1.4 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.


from __future__ import print_function, absolute_import # NEED TO STAY AS TOP IMPORT
import sys
import os
import os.path


class pypeBatch(object):

    def __init__(self):
        self.Arguments = []

        self.Directory = '.'
        self.ScriptName = ''
        self.ScriptArguments = []

    def GetUsageString(self):
        usageString = 'Usage: pypebatch -d directory scriptName scriptArguments ["pythonStringMethods"]'
        return usageString

    def PrintLog(self,logMessage,indent=0):
        indentUnit = '    '
        indentation = ''
        for i in range(indent):
            indentation = indentation + indentUnit
        print(indentation + logMessage)

    def PrintError(self,logMessage):
        print(logMessage)

    def ParseArguments(self):
        state = 0
        for arg in self.Arguments:
            if (arg == '--help'):
                self.PrintLog(self.GetUsageString())
                sys.exit()
            elif (arg == '-d'):
                state = 1
            elif (state == 1):
                self.Directory = arg
                state = 2
            elif (state == 0):
                state == 2
            elif (state == 2):
                if (self.ScriptName == ''):
                    self.ScriptName = arg
                self.ScriptArguments.append(arg)

    def ReplaceFileNamesInScriptArguments(self,fileName):
        actualScriptArguments = []
        for arg in self.ScriptArguments:
            if not ((arg[0]=='[') & (arg[-1]==']')):
                actualScriptArguments.append(arg)
                continue
            pattern = arg[1:-1]
            actualArgument = self.Directory + '/'
            if (pattern != ''):
##                exec('actualArgument += fileName.' + pattern)
                actualArgument += fileName.__getattribute__(pattern)
            else:
                actualArgument += fileName
            actualScriptArguments.append(actualArgument)
        return actualScriptArguments

    def Execute(self):
        self.PrintLog('')
        moduleName = self.ScriptName
        exec('from vmtk import '+ moduleName)
        scriptObjectClassName = ''
        exec ('scriptObjectClassName =  '+moduleName+'.'+moduleName)
        moduleScriptObjectClassName = moduleName+'.'+scriptObjectClassName
        scriptObject = 0

        fileNames = os.listdir(self.Directory)

        for fileName in fileNames:

            self.PrintLog('Creating ' + scriptObjectClassName + ' instance.')
            exec ('scriptObject = '+moduleScriptObjectClassName+'()')
            completeFileName = os.path.normpath(self.Directory + '/' + fileName)
            self.PrintLog('Replacing FileNames in ' + scriptObject.ScriptName + ' arguments')
            actualScriptArguments = self.ReplaceFileNamesInScriptArguments(fileName)
            scriptObject.Arguments = actualScriptArguments
            scriptObject.ParseArguments()
            scriptString = ''
            for arg in actualScriptArguments:
                scriptString += arg + ' '
            self.PrintLog('Executing ' + scriptString)
            scriptObject.Execute()


if __name__=='__main__':
    batch = pypeBatch()
    batch.Arguments = sys.argv
    batch.ParseArguments()
    batch.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtknetworkextraction.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
from vmtk import vtkvmtk
import vtk
import sys

from vmtk import pypes


class vmtkNetworkExtraction(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Network = None
        self.GraphLayout = None

        self.RadiusArrayName = 'Radius'
        self.TopologyArrayName = 'Topology'
        self.MarksArrayName = 'Marks'

        self.AdvancementRatio = 1.05

        self.SetScriptName('vmtknetworkextraction')
        self.SetScriptDoc('extract a network of approximated centerlines from a surface, the surface must have at least an opening')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['AdvancementRatio','advancementratio','float',1,'(1.0,)','the ratio between the sphere step and the local maximum radius'],
            ['RadiusArrayName','radiusarray','str',1],
            ['TopologyArrayName','topologyarray','str',1],
            ['MarksArrayName','marksarray','str',1]
            ])
        self.SetOutputMembers([
            ['Network','o','vtkPolyData',1,'','the output network','vmtknetworkwriter'],
            ['GraphLayout','ograph','vtkPolyData',1,'','the output graph layout','vmtknetworkwriter']
            ])

    def PrintProgress(self,obj,event):
        self.OutputProgress(obj.GetProgress(),10)

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        networkExtraction = vtkvmtk.vtkvmtkPolyDataNetworkExtraction()
        networkExtraction.SetInputData(self.Surface)
        networkExtraction.SetAdvancementRatio(self.AdvancementRatio)
        networkExtraction.SetRadiusArrayName(self.RadiusArrayName)
        networkExtraction.SetTopologyArrayName(self.TopologyArrayName)
        networkExtraction.SetMarksArrayName(self.MarksArrayName)
        networkExtraction.AddObserver("ProgressEvent", self.PrintProgress)
        networkExtraction.Update()

        self.EndProgress()

        self.Network = networkExtraction.GetOutput()
        self.GraphLayout = networkExtraction.GetGraphLayout()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfaceviewer.py,v $
## Language:  Python
## Date:      $Date: 2006/05/26 12:35:13 $
## Version:   $Revision: 1.10 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vmtkrenderer
from vmtk import pypes


class vmtkSurfaceViewer(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.Display = 1
        self.Opacity = 1.0
        self.ArrayName = ''
        self.ScalarRange = [0.0, 0.0]
        self.ColorMap = 'cooltowarm'
        self.NumberOfColors = 256
        self.Legend = 0
        self.LegendTitle = ''
        self.Grayscale = 0
        self.FlatInterpolation = 0
        self.DisplayCellData = 0
        self.Color = [-1.0, -1.0, -1.0]
        self.LineWidth = 1
        self.Representation = 'surface'
        self.DisplayTag = False
        self.RegionTagArrayName = 'RegionTagArray'
        self.NumberOfRegions = 0
        self.TagSet = []

        self.Actor = None
        self.ScalarBarActor = None

        self.SetScriptName('vmtksurfaceviewer')
        self.SetScriptDoc('display a surface')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer'],
            ['Display','display','bool',1,'','toggle rendering'],
            ['Representation','representation','str',1,'["surface","wireframe","edges"]','change surface representation'],
            ['Opacity','opacity','float',1,'(0.0,1.0)','object opacity in the scene'],
            ['ArrayName','array','str',1,'','name of the array where the scalars to be displayed are stored'],
            ['ScalarRange','scalarrange','float',2,'','range of the scalar map'],
            ['ColorMap','colormap','str',1,'["rainbow","blackbody","cooltowarm","grayscale"]','choose the color map'],
            ['NumberOfColors','numberofcolors','int',1,'','number of colors in the color map'],
            ['Legend','legend','bool',1,'','toggle scalar bar'],
            ['FlatInterpolation','flat','bool',1,'','toggle flat or shaded surface display'],
            ['DisplayCellData','celldata','bool',1,'','toggle display of point or cell data'],
            ['DisplayTag','displaytag','bool',1,'','toggle rendering of tag'],
            ['RegionTagArrayName','regiontagarray','str',1,'','name of the array where the tags to be displayed are stored'],
            ['Color','color','float',3,'','RGB color of the object in the scene'],
            ['LineWidth','linewidth','int',1,'(0.0,)','width of line objects in the scene'],
            ['LegendTitle','legendtitle','str',1,'','title of the scalar bar']])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter'],
            ['Actor','oactor','vtkActor',1,'','the output actor']
            ])

    def SetSurfaceRepresentation(self, representation):
        if representation == 'surface':
            self.Actor.GetProperty().SetRepresentationToSurface()
            self.Actor.GetProperty().EdgeVisibilityOff()
        elif representation == 'edges':
            self.Actor.GetProperty().SetRepresentationToSurface()
            self.Actor.GetProperty().EdgeVisibilityOn()
        elif representation == 'wireframe':
            self.Actor.GetProperty().SetRepresentationToWireframe()
            self.Actor.GetProperty().EdgeVisibilityOff()
        self.Representation = representation

    def RepresentationCallback(self, obj):
        if not self.Actor:
            return

        if self.Representation == 'surface':
            representation = 'edges'
        elif self.Representation == 'edges':
            representation = 'wireframe'
        elif self.Representation == 'wireframe':
            representation = 'surface'

        self.SetSurfaceRepresentation(representation)

        self.vmtkRenderer.RenderWindow.Render()

    def BuildView(self):

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self)

        if self.Actor:
            self.vmtkRenderer.Renderer.RemoveActor(self.Actor)

        if self.ScalarBarActor:
            self.vmtkRenderer.Renderer.RemoveActor(self.ScalarBarActor)

        if self.Surface:
            mapper = vtk.vtkPolyDataMapper()
            mapper.SetInputData(self.Surface)
            if self.ArrayName:
                if self.DisplayCellData == 0:
                    self.Surface.GetPointData().SetActiveScalars(self.ArrayName)
                    array = self.Surface.GetPointData().GetScalars()
                else:
                    self.Surface.GetCellData().SetActiveScalars(self.ArrayName)
                    array = self.Surface.GetCellData().GetScalars()
                    mapper.SetScalarModeToUseCellData()
                if (self.ScalarRange[1] > self.ScalarRange[0]):
                    mapper.SetScalarRange(self.ScalarRange)
                elif array:
                    mapper.SetScalarRange(array.GetRange(0))
                if self.Grayscale:
                    lut = vtk.vtkLookupTable()
                    lut.SetValueRange(0.0,1.0)
                    lut.SetSaturationRange(0.0,0.0)
                    mapper.SetLookupTable(lut)
            else:
                mapper.ScalarVisibilityOff()

            if self.ColorMap == 'grayscale':
                lut = mapper.GetLookupTable()
                lut.SetNumberOfTableValues(self.NumberOfColors)
                lut.SetValueRange(0.0,1.0)
                lut.SetSaturationRange(0.0,0.0)
                lut.Build()
                mapper.SetLookupTable(lut)

            if self.ColorMap == 'rainbow':
                lut = mapper.GetLookupTable()
                lut.SetHueRange(0.666667,0.0)
                lut.SetSaturationRange(0.75,0.75)
                lut.SetValueRange(1.0,1.0)
                lut.SetAlphaRange(1.0,1.0)
                lut.SetNumberOfColors(self.NumberOfColors)
                lut.Build()
                mapper.SetLookupTable(lut)

            if self.ColorMap == 'blackbody':
               lut = mapper.GetLookupTable()
               lut.SetNumberOfTableValues(self.NumberOfColors)
               colorTransferFunction = vtk.vtkColorTransferFunction()
               colorTransferFunction.SetColorSpaceToRGB()
               colorTransferFunction.AddRGBPoint(0,0.0,0.0,0.0)
               colorTransferFunction.AddRGBPoint(0.4,0.901961,0.0,0.0)
               colorTransferFunction.AddRGBPoint(0.8,0.901961,0.901961,0.0)
               colorTransferFunction.AddRGBPoint(1.0,1.0,1.0,1.0)
               for ii,ss in enumerate([float(xx)/float(self.NumberOfColors) for xx in range(self.NumberOfColors)]):
                   cc = colorTransferFunction.GetColor(ss)
                   lut.SetTableValue(ii,cc[0],cc[1],cc[2],1.0)
               lut.Build()
               mapper.SetLookupTable(lut)

            if self.ColorMap == 'cooltowarm':
               lut = mapper.GetLookupTable()
               lut.SetNumberOfTableValues(self.NumberOfColors)
               colorTransferFunction = vtk.vtkColorTransferFunction()
               colorTransferFunction.SetColorSpaceToDiverging()
               colorTransferFunction.AddRGBPoint(0,0.231373,0.298039,0.752941)
               colorTransferFunction.AddRGBPoint(0.5,0.865003,0.865003,0.865003)
               colorTransferFunction.AddRGBPoint(1.0,0.705882,0.0156863,0.14902)
               for ii,ss in enumerate([float(xx)/float(self.NumberOfColors) for xx in range(self.NumberOfColors)]):
                   cc = colorTransferFunction.GetColor(ss)
                   lut.SetTableValue(ii,cc[0],cc[1],cc[2],1.0)
               lut.Build()
               mapper.SetLookupTable(lut)

            self.Actor = vtk.vtkActor()
            self.Actor.SetMapper(mapper)
            if (self.Color[0] >= 0.0):
                self.Actor.GetProperty().SetColor(self.Color)
            self.Actor.GetProperty().SetOpacity(self.Opacity)
            self.Actor.GetProperty().SetLineWidth(self.LineWidth)
            if self.FlatInterpolation:
                self.Actor.GetProperty().SetInterpolationToFlat()
            self.SetSurfaceRepresentation(self.Representation)
            self.vmtkRenderer.Renderer.AddActor(self.Actor)
            self.vmtkRenderer.AddKeyBinding('w','Change surface representation.',self.RepresentationCallback)

        if self.Legend and self.Actor:
            self.ScalarBarActor = vtk.vtkScalarBarActor()
            self.ScalarBarActor.SetLookupTable(self.Actor.GetMapper().GetLookupTable())
            self.ScalarBarActor.GetLabelTextProperty().ItalicOff()
            self.ScalarBarActor.GetLabelTextProperty().BoldOff()
            self.ScalarBarActor.GetLabelTextProperty().ShadowOff()
##             self.ScalarBarActor.GetLabelTextProperty().SetColor(0.0,0.0,0.0)
            self.ScalarBarActor.SetLabelFormat('%.2f')
            self.ScalarBarActor.SetTitle(self.LegendTitle)
            self.vmtkRenderer.Renderer.AddActor(self.ScalarBarActor)

        if self.Display:
            self.vmtkRenderer.Render()
##            self.vmtkRenderer.Renderer.RemoveActor(self.Actor)
##            self.vmtkRenderer.Renderer.RemoveActor(self.ScalarBarActor)

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()

    def BuildViewWithTag(self):

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self)

        if self.Actor:
            self.vmtkRenderer.Renderer.RemoveActor(self.Actor)

        if self.ScalarBarActor:
            self.vmtkRenderer.Renderer.RemoveActor(self.ScalarBarActor)

        if self.Surface.GetPointData().GetArray(self.RegionTagArrayName) == None and self.Surface.GetCellData().GetArray(self.RegionTagArrayName) == None:
            self.PrintError('Error: no regiontagarray with name specified')

        elif self.Surface.GetPointData().GetArray(self.RegionTagArrayName) != None:

            regionTagArray = self.Surface.GetPointData().GetArray(self.RegionTagArrayName)
            for j in range (self.Surface.GetNumberOfPoints()):
                if regionTagArray.GetTuple1(j) not in self.TagSet:
                    self.TagSet.append(regionTagArray.GetTuple1(j))
            self.TagSet.sort()
            self.NumberOfRegions = len(self.TagSet)

            tagSetCopy = list(self.TagSet)
            labelPoints = vtk.vtkPoints()
            labelPoints.SetNumberOfPoints(len(self.TagSet))
            point = [0.0,0.0,0.0]
            for j in range (self.Surface.GetNumberOfPoints()):
                item = regionTagArray.GetTuple1(j)
                if item in tagSetCopy:
                    self.Surface.GetPoint(j, point)
                    labelPoints.SetPoint(self.TagSet.index(item), point)
                    tagSetCopy.remove(item)

            self.Surface.GetPointData().SetActiveScalars(self.RegionTagArrayName)

        elif self.Surface.GetCellData().GetArray(self.RegionTagArrayName) != None:

            regionTagArray = self.Surface.GetCellData().GetArray(self.RegionTagArrayName)
            for j in range (self.Surface.GetNumberOfCells()):
                if regionTagArray.GetTuple1(j) not in self.TagSet:
                    self.TagSet.append(regionTagArray.GetTuple1(j))
            self.TagSet.sort()
            self.NumberOfRegions = len(self.TagSet)

            tagSetCopy = list(self.TagSet)
            labelPoints = vtk.vtkPoints()
            labelPoints.SetNumberOfPoints(len(self.TagSet))
            point = [0.0,0.0,0.0]

            cellCenters = vtk.vtkCellCenters()
            cellCenters.SetInputData(self.Surface)
            cellCenters.Update()

            regionTagArrayCenters = cellCenters.GetOutput().GetPointData().GetArray(self.RegionTagArrayName)

            for j in range (cellCenters.GetOutput().GetNumberOfPoints()):
                item = regionTagArrayCenters.GetTuple1(j)
                if item in tagSetCopy:
                    cellCenters.GetOutput().GetPoint(j, point)
                    labelPoints.SetPoint(self.TagSet.index(item), point)
                    tagSetCopy.remove(item)

            self.Surface.GetCellData().SetActiveScalars(self.RegionTagArrayName)

        labelPolyData = vtk.vtkPolyData()
        labelPolyData.SetPoints(labelPoints)

        labelArray = vtk.vtkIntArray()
        labelArray.SetNumberOfComponents(1)
        labelArray.SetNumberOfTuples(self.NumberOfRegions)
        labelArray.SetName('label')
        labelArray.FillComponent(0,0)

        labelPolyData.GetPointData().AddArray(labelArray)

        for item in self.TagSet:
            labelArray.SetTuple1(self.TagSet.index(item), item)

        labelPolyData.GetPointData().SetActiveScalars('label')
        labelsMapper = vtk.vtkLabeledDataMapper()
        labelsMapper.SetInputData(labelPolyData)
        labelsMapper.SetLabelModeToLabelScalars()
        labelsMapper.GetLabelTextProperty().SetColor(1, 1, 1)
        labelsMapper.GetLabelTextProperty().SetFontSize(14)
        self.labelsActor = vtk.vtkActor2D()
        self.labelsActor.SetMapper(labelsMapper)
        self.vmtkRenderer.Renderer.AddActor(self.labelsActor)

        surfaceMapper = vtk.vtkPolyDataMapper()
        surfaceMapper.SetInputData(self.Surface)
        surfaceMapper.ScalarVisibilityOn()
        surfaceMapper.SetScalarRange(self.TagSet[0], self.TagSet[self.NumberOfRegions-1])
        self.Actor = vtk.vtkActor()
        self.Actor.SetMapper(surfaceMapper)
        self.Actor.GetProperty().SetOpacity(self.Opacity)
        self.Actor.GetProperty().SetLineWidth(self.LineWidth)
        if self.FlatInterpolation:
            self.Actor.GetProperty().SetInterpolationToFlat()
        self.SetSurfaceRepresentation(self.Representation)
        self.vmtkRenderer.Renderer.AddActor(self.Actor)
        self.vmtkRenderer.AddKeyBinding('w','Change surface representation.',self.RepresentationCallback)

        if self.Legend and self.Actor:
            self.ScalarBarActor = vtk.vtkScalarBarActor()
            self.ScalarBarActor.SetLookupTable(self.Actor.GetMapper().GetLookupTable())
            self.ScalarBarActor.GetLabelTextProperty().ItalicOff()
            self.ScalarBarActor.GetLabelTextProperty().BoldOff()
            self.ScalarBarActor.GetLabelTextProperty().ShadowOff()
##             self.ScalarBarActor.GetLabelTextProperty().SetColor(0.0,0.0,0.0)
            self.ScalarBarActor.SetLabelFormat('%.2f')
            self.ScalarBarActor.SetTitle(self.LegendTitle)
            self.vmtkRenderer.Renderer.AddActor(self.ScalarBarActor)

        if self.Display:
            self.vmtkRenderer.Render()
##            self.vmtkRenderer.Renderer.RemoveActor(self.Actor)
##            self.vmtkRenderer.Renderer.RemoveActor(self.ScalarBarActor)

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()

    def Execute(self):
        if (not self.Surface) and self.Display:
            self.PrintError('Error: no Surface.')

        if not self.DisplayTag:
            self.BuildView()
        else:
            self.BuildViewWithTag()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



"""Build PyPI packages (sdist and binary wheels) for vmtk.

The binary wheel is designed to work alongside the "vtk" wheels published on
PyPI: vmtk's C++ classes are compiled against the VTK wheel SDK that Kitware
publishes for every VTK release (the same SDK that VTK remote modules such as
VTKU3DExporter build against, see
https://github.com/KitwareMedical/VTKExternalModule and
https://github.com/Korijn/VTKU3DExporter for the approach this file is
modeled on). ITK does not provide an equivalent SDK, so ITK is built from
source once (outside the per-Python build trees, so it is reused when
building wheels for several Python versions) and linked in: statically on
Windows, as shared libraries on Linux/macOS where the wheel repair step
(auditwheel/delocate) bundles the ITK runtime libraries into the wheel.

Environment variables understood by this script:

  VMTK_VTK_VERSION        VTK version to build against and to pin as the
                          install requirement (default: 9.6.2). A wheel SDK
                          must exist for this version at vtk.org.
  VTK_WHEEL_SDK_PATH      Use an already downloaded/unpacked VTK wheel SDK
                          instead of downloading one.
  VMTK_ITK_VERSION        ITK version to build (default: 5.4.5).
  ITK_DIR                 Use an existing ITK build/install tree (directory
                          containing ITKConfig.cmake) instead of building
                          ITK from source.
  VMTK_ITK_SHARED         ON/OFF, override the default ITK library type
                          (default: OFF on Windows, ON elsewhere; a single
                          shared ITK is required on Linux/macOS so that all
                          vmtk libraries share one ITK object-factory
                          registry).
  VMTK_BUILD_TETGEN       ON/OFF (default OFF for PyPI packages: TetGen
                          1.4.3 is distributed under its own license terms
                          that differ from vmtk's BSD-style license, so it
                          is not included in packages published on PyPI;
                          the conda packages build it).
  Python3_EXECUTABLE      Python interpreter to build against (default:
                          sys.executable).
  VMTK_VERSION_SUFFIX     Pre-release suffix appended to the version read
                          from CMakeLists.txt (e.g. "b1" -> 1.5.0b1);
                          overrides the set(VMTK_VERSION_SUFFIX ...) value
                          in CMakeLists.txt.
"""

import os
import platform
import subprocess
import sys
import tarfile
import urllib.request
from pathlib import Path

from skbuild import setup


DEFAULT_VTK_VERSION = "9.6.2"
DEFAULT_ITK_VERSION = "5.4.5"

repo_root = Path(__file__).parent.resolve()
deps_dir = repo_root / "_deps"


def vmtk_version():
    """Read the vmtk version from the top-level CMakeLists.txt.

    An optional pre-release suffix (PEP 440: "a1", "b2", "rc1", ".dev0",
    ...) is appended from set(VMTK_VERSION_SUFFIX "...") or, taking
    precedence, from the VMTK_VERSION_SUFFIX environment variable, e.g.
    1.5.0 -> 1.5.0b1. Pre-releases can be uploaded to PyPI as usual; pip
    only installs them when explicitly requested (--pre or an exact pin).
    """
    version = {}
    cmakelists = (repo_root / "CMakeLists.txt").read_text(encoding="utf-8")
    for component in ("MAJOR", "MINOR", "PATCH"):
        marker = f"set(VMTK_VERSION_{component} "
        line = next(l for l in cmakelists.splitlines() if marker in l)
        version[component] = line.split(marker)[1].split(")")[0].strip()

    suffix = os.getenv("VMTK_VERSION_SUFFIX")
    if suffix is None:
        marker = "set(VMTK_VERSION_SUFFIX "
        line = next((l for l in cmakelists.splitlines() if marker in l), "")
        if line:
            suffix = line.split(marker)[1].split(")")[0].strip().strip('"')
        else:
            suffix = ""

    return "{MAJOR}.{MINOR}.{PATCH}".format(**version) + suffix


def vtk_version():
    return os.getenv("VMTK_VTK_VERSION", DEFAULT_VTK_VERSION)


def itk_version():
    return os.getenv("VMTK_ITK_VERSION", DEFAULT_ITK_VERSION)


def python_tag():
    major, minor = sys.version_info[:2]
    return f"cp{major}{minor}-cp{major}{minor}"


def vtk_wheel_sdk_platform_suffix():
    machine = platform.machine().lower()
    if sys.platform == "linux":
        if machine == "aarch64":
            return "manylinux_2_28_aarch64"
        return "manylinux2014_x86_64.manylinux_2_17_x86_64"
    if sys.platform == "darwin":
        is_arm = (
            machine == "arm64"
            # ARCHFLAGS: see https://github.com/pypa/cibuildwheel/discussions/997
            or os.getenv("ARCHFLAGS") == "-arch arm64"
        )
        return "macosx_11_0_arm64" if is_arm else "macosx_10_10_x86_64"
    if sys.platform == "win32":
        return "win_amd64"
    raise NotImplementedError(sys.platform)


def vtk_wheel_sdk_name():
    return (
        f"vtk-wheel-sdk-{vtk_version()}-{python_tag()}"
        f"-{vtk_wheel_sdk_platform_suffix()}"
    )


def vtk_wheel_sdk_url():
    version = vtk_version()
    major_minor = ".".join(version.split(".")[:2])
    if tuple(int(x) for x in version.split(".")[:2]) >= (9, 4):
        # Starting with VTK 9.4 the wheel SDKs are published in the release
        # file area.
        base_url = f"https://vtk.org/files/release/{major_minor}/"
    else:
        base_url = "https://vtk.org/files/wheel-sdks/"
    return f"{base_url}{vtk_wheel_sdk_name()}.tar.xz"


def auto_download_vtk_wheel_sdk():
    """Download and unpack the VTK wheel SDK matching the current Python.

    Returns the path to the unpacked SDK. A previously unpacked SDK is
    reused.
    """
    install_path = deps_dir / vtk_wheel_sdk_name()
    if install_path.exists():
        return install_path

    url = vtk_wheel_sdk_url()
    archive_path = install_path.with_suffix(".tar.xz.tmp")
    install_path.parent.mkdir(parents=True, exist_ok=True)
    print(f"Downloading VTK wheel SDK: {url}")
    urllib.request.urlretrieve(url, archive_path)

    partial_path = install_path.with_name(install_path.name + ".extracting")
    with tarfile.open(archive_path) as tar:
        tar.extractall(partial_path)
    archive_path.unlink()
    # The archive contains a single top-level directory; normalize so that
    # the SDK content lives directly in install_path.
    entries = list(partial_path.iterdir())
    if len(entries) == 1 and entries[0].is_dir():
        entries[0].rename(install_path)
        partial_path.rmdir()
    else:
        partial_path.rename(install_path)
    return install_path


def vtk_cmake_dir(sdk_path):
    """Locate the VTK CMake package directory inside the wheel SDK."""
    matches = list(Path(sdk_path).glob("**/headers/cmake"))
    if len(matches) != 1:
        raise RuntimeError(
            f"Unable to find a unique headers/cmake directory in {sdk_path}"
        )
    return matches[0]


def linux_needs_old_cxx11_abi():
    """The x86_64 VTK wheels (and their SDK) are built on manylinux2014,
    whose toolchain uses the pre-cxx11 libstdc++ ABI
    (_GLIBCXX_USE_CXX11_ABI=0). Anything linking against them must use the
    same ABI or std::string-based VTK symbols do not resolve at import
    time. The aarch64 wheels are built on manylinux_2_28 with the default
    (new) ABI, so no flag is needed there."""
    return (
        sys.platform == "linux"
        and "manylinux2014" in vtk_wheel_sdk_platform_suffix()
    )


def itk_use_shared_libs():
    override = os.getenv("VMTK_ITK_SHARED")
    if override:
        return override.upper() in ("ON", "1", "TRUE", "YES")
    # A single shared ITK is required on Linux/macOS: with static ITK every
    # vmtk shared library embeds its own copy of ITK's object-factory
    # registry and image IO stops working ("There are no registered IO
    # factories"). Windows was validated with static ITK, which avoids
    # having to bundle and locate ITK DLLs.
    return sys.platform != "win32"


def auto_build_itk():
    """Clone and build ITK once, outside the per-Python build tree.

    Returns the ITK build directory (contains ITKConfig.cmake). The build
    is reused across Python versions (the ITK Python wrapping is not built,
    so the result is Python independent) and by CI caching.
    """
    version = itk_version()
    shared = itk_use_shared_libs()
    lib_type = "shared" if shared else "static"
    # ITK must be built with the same libstdc++ ABI as vmtk and VTK
    # (std::string crosses the ITK API boundary); encode the ABI in the
    # build directory name so that cached builds with the other ABI are
    # not reused.
    abi_suffix = "-abi0" if linux_needs_old_cxx11_abi() else ""
    source_dir = deps_dir / f"itk-src-{version}"
    build_dir = deps_dir / f"itk-build-{version}-{lib_type}{abi_suffix}"
    stamp = build_dir / ".vmtk-itk-build-complete"

    # The source tree must exist even when a cached build tree is reused:
    # the build-tree ITKConfig.cmake references CMake modules and headers
    # inside the source tree (CI caches only the build tree).
    if not (source_dir / "CMakeLists.txt").exists():
        subprocess.check_call(
            [
                "git", "clone",
                "--depth", "1",
                "--branch", f"v{version}",
                "https://github.com/InsightSoftwareConsortium/ITK.git",
                str(source_dir),
            ]
        )

    if stamp.exists():
        return build_dir

    configure_cmd = [
        "cmake",
        "-S", str(source_dir),
        "-B", str(build_dir),
        f"-DBUILD_SHARED_LIBS:BOOL={'ON' if shared else 'OFF'}",
        "-DCMAKE_POSITION_INDEPENDENT_CODE:BOOL=ON",
        "-DCMAKE_BUILD_TYPE:STRING=Release",
        "-DBUILD_TESTING:BOOL=OFF",
        "-DBUILD_EXAMPLES:BOOL=OFF",
        "-DITK_LEGACY_SILENT:BOOL=ON",
    ]
    if linux_needs_old_cxx11_abi():
        configure_cmd.append(
            "-DCMAKE_CXX_FLAGS:STRING=-D_GLIBCXX_USE_CXX11_ABI=0"
        )
    if sys.platform != "win32":
        # Ninja is provided by the build requirements (pyproject.toml). On
        # Windows the default (Visual Studio) generator is used instead,
        # because ninja would additionally require the MSVC environment
        # variables that scikit-build only sets up for the main build.
        configure_cmd += ["-G", "Ninja"]
    subprocess.check_call(configure_cmd)
    subprocess.check_call(
        [
            "cmake",
            "--build", str(build_dir),
            "--config", "Release",
            "--parallel", str(os.cpu_count() or 2),
        ]
    )
    stamp.touch()
    return build_dir


def native_build_needed():
    """Return False for commands that do not run the CMake build (sdist,
    metadata generation, ...), so that the VTK wheel SDK download and the
    ITK build are only triggered when actually compiling."""
    argv = sys.argv[1:]
    native_commands = {
        "build", "build_ext", "bdist", "bdist_wheel", "install", "develop",
        "editable_wheel",
    }
    if any(arg in native_commands for arg in argv):
        return True
    non_native_commands = {
        "sdist", "egg_info", "dist_info", "clean", "check",
        "--name", "--version", "--fullname",
    }
    if any(arg in non_native_commands for arg in argv):
        return False
    return True


cmake_args = [
    "-DVMTK_USE_SUPERBUILD:BOOL=OFF",
    "-DVMTK_WHEEL_BUILD:BOOL=ON",
    "-DUSE_SYSTEM_VTK:BOOL=ON",
    "-DUSE_SYSTEM_ITK:BOOL=ON",
    "-DVTK_VMTK_WRAP_PYTHON:BOOL=ON",
    "-DBUILD_SHARED_LIBS:BOOL=ON",
    "-DVMTK_BUILD_TESTING:BOOL=OFF",
    "-DVMTK_USE_RENDERING:BOOL=ON",
    "-DVTK_VMTK_CONTRIB:BOOL=ON",
    "-DVMTK_CONTRIB_SCRIPTS:BOOL=ON",
    "-DVMTK_SCRIPTS_ENABLED:BOOL=ON",
    # TetGen is excluded from PyPI packages because of its license terms
    # (set VMTK_BUILD_TETGEN=ON to build it in for private use).
    "-DVMTK_BUILD_TETGEN:BOOL=" + os.getenv("VMTK_BUILD_TETGEN", "OFF"),
    "-DVTK_VMTK_BUILD_TETGEN:BOOL=" + os.getenv("VMTK_BUILD_TETGEN", "OFF"),
]

if native_build_needed():
    # Resolve VTK (wheel SDK) and ITK.
    vtk_sdk_path = os.getenv("VTK_WHEEL_SDK_PATH")
    if vtk_sdk_path is None:
        vtk_sdk_path = auto_download_vtk_wheel_sdk()

    vtk_dir = vtk_cmake_dir(vtk_sdk_path)

    itk_dir = os.getenv("ITK_DIR")
    if itk_dir is None:
        itk_dir = auto_build_itk()

    python3_executable = os.getenv("Python3_EXECUTABLE", sys.executable)

    cmake_args += [
        f"-DVTK_DIR:PATH={Path(vtk_dir).as_posix()}",
        f"-DITK_DIR:PATH={Path(itk_dir).as_posix()}",
        f"-DPython3_EXECUTABLE:FILEPATH={Path(python3_executable).as_posix()}",
    ]

    if linux_needs_old_cxx11_abi():
        cmake_args.append(
            "-DCMAKE_CXX_FLAGS:STRING=-D_GLIBCXX_USE_CXX11_ABI=0"
        )

    if itk_use_shared_libs():
        # Lets the wheel repair tools (auditwheel/delocate) and locally
        # built wheels resolve the shared ITK runtime libraries via an
        # rpath entry.
        itk_runtime_lib_dir = Path(itk_dir) / "lib"
        cmake_args.append(
            f"-DVMTK_WHEEL_ITK_RUNTIME_LIB_DIR:PATH={itk_runtime_lib_dir.as_posix()}"
        )

long_description = (repo_root / "README.md").read_text(encoding="utf-8")

setup(
    name="vmtk",
    version=vmtk_version(),
    description="vmtk - the Vascular Modeling Toolkit",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://vmtk.github.io",
    author="Luca Antiga, David Steinman, Simone Manini, Richard Izzo",
    license="BSD",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Medical Science Apps.",
    ],
    keywords="vascular modeling mesh segmentation centerline vtk itk",
    # The whole package content is produced by the CMake build (see the
    # VMTK_WHEEL_BUILD option in the top-level CMakeLists.txt, which makes
    # the install rules place everything into a "vmtk" directory).
    # scikit-build assigns the CMake-installed files to the declared "vmtk"
    # package because the package_dir entry matches the install destination;
    # the placeholder vmtk/.keep file in the source tree only exists so that
    # the declared package directory is present (the same pattern is used by
    # VTKU3DExporter).
    packages=["vmtk"],
    package_dir={"vmtk": "vmtk"},
    include_package_data=False,
    cmake_args=cmake_args,
    python_requires=">=3.12",
    install_requires=[
        f"vtk=={vtk_version()}",
        "numpy",
    ],
    extras_require={
        # Optional runtime dependencies: joblib parallelizes
        # vmtkcenterlinesnetwork (serial fallback without it), h5py is
        # imported on demand by the numpy reader/writer scripts.
        "all": ["h5py", "joblib"],
    },
    entry_points={
        "console_scripts": [
            "vmtk = vmtk.pypemain:main",
        ],
    },
    project_urls={
        "Bug Reports": "https://github.com/vmtk/vmtk/issues",
        "Source": "https://github.com/vmtk/vmtk",
        "Documentation": "http://vmtk.github.io/documentation",
    },
)



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkbifurcationprofiles.py,v $
## Language:  Python
## Date:      $Date: 2006/10/17 15:16:16 $
## Version:   $Revision: 1.1 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkBifurcationProfiles(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Centerlines = None
        self.BifurcationProfiles = None

        self.RadiusArrayName = 'MaximumInscribedSphereRadius'
        self.GroupIdsArrayName = 'GroupIds'
        self.CenterlineIdsArrayName = 'CenterlineIds'
        self.TractIdsArrayName = 'TractIds'
        self.BlankingArrayName = 'Blanking'

        self.BifurcationProfileGroupIdsArrayName = 'BifurcationProfileGroupIds'
        self.BifurcationProfileBifurcationGroupIdsArrayName = 'BifurcationProfileBifurcationGroupIds'
        self.BifurcationProfileOrientationArrayName = 'BifurcationProfileOrientation'

        self.SetScriptName('vmtkbifurcationprofiles')
        self.SetScriptDoc('compute bifurcation profiles, i.e. the bifurcation splitting lines. The script takes in input the surface and the relative centerlines, both already split into branches.')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface, already split into branches','vmtksurfacereader'],
            ['Centerlines','centerlines','vtkPolyData',1,'','the input centerlines, already split into branches','vmtksurfacereader'],
            ['RadiusArrayName','radiusarray','str',1,'','name of the array where centerline radius is stored'],
            ['GroupIdsArrayName','groupidsarray','str',1,'','name of the array where centerline group ids are stored'],
            ['CenterlineIdsArrayName','centerlineidsarray','str',1,'','name of the array where centerline ids are stored'],
            ['TractIdsArrayName','tractidsarray','str',1,'','name of the array where centerline tract ids are stored'],
            ['BlankingArrayName','blankingarray','str',1,'','name of the array where centerline blanking information about branches is stored'],
            ['BifurcationProfileGroupIdsArrayName','bifurcationprofilegroupids','str',1,'','name of the array where the group id to which each profile belongs has to be stored'],
            ['BifurcationProfileBifurcationGroupIdsArrayName','bifurcationprofilebifurcationgroupids','str',1,'','name of the array where the bifurcation group id to which each profile belongs has to be stored'],
            ['BifurcationProfileOrientationArrayName','bifurcationprofileorientation','str',1,'','name of the array containing 0 if a profile is upstream and 0 downstream its bifurcation']
            ])
        self.SetOutputMembers([
            ['BifurcationProfiles','o','vtkPolyData',1,'','the output sections','vmtksurfacewriter'],
            ['BifurcationProfileGroupIdsArrayName','bifurcationprofilegroupids','str',1,'','name of the array where the group id to which each profile belongs are stored'],
            ['BifurcationProfileBifurcationGroupIdsArrayName','bifurcationprofilebifurcationgroupids','str',1,'','name of the array where the bifurcation group id to which each profile belongs has to be stored'],
            ['BifurcationProfileOrientationArrayName','bifurcationprofileorientation','str',1,'','name of the array containing 0 if a profile is upstream and 0 downstream its bifurcation']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        bifurcationProfiles = vtkvmtk.vtkvmtkPolyDataBifurcationProfiles()
        bifurcationProfiles.SetInputData(self.Surface)
        bifurcationProfiles.SetGroupIdsArrayName(self.GroupIdsArrayName)
        bifurcationProfiles.SetCenterlines(self.Centerlines)
        bifurcationProfiles.SetCenterlineRadiusArrayName(self.RadiusArrayName)
        bifurcationProfiles.SetCenterlineGroupIdsArrayName(self.GroupIdsArrayName)
        bifurcationProfiles.SetCenterlineIdsArrayName(self.CenterlineIdsArrayName)
        bifurcationProfiles.SetCenterlineTractIdsArrayName(self.TractIdsArrayName)
        bifurcationProfiles.SetBlankingArrayName(self.BlankingArrayName)
        bifurcationProfiles.SetBifurcationProfileGroupIdsArrayName(self.BifurcationProfileGroupIdsArrayName)
        bifurcationProfiles.SetBifurcationProfileBifurcationGroupIdsArrayName(self.BifurcationProfileBifurcationGroupIdsArrayName)
        bifurcationProfiles.SetBifurcationProfileOrientationArrayName(self.BifurcationProfileOrientationArrayName)
        bifurcationProfiles.Update()

        self.BifurcationProfiles = bifurcationProfiles.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkpolyballmodeller.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.7 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys
from vmtk import vtkvmtk
from vmtk import pypes


class vmtkPolyBallModeller(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.ReferenceImage = None
        self.RadiusArrayName = None
        self.Image = None
        self.ModelBounds = None
        self.SampleDimensions = [64,64,64]
        self.NegateFunction = 0

        self.SetScriptName('vmtkpolyballmodeller')
        self.SetScriptDoc('converts a polyball to an image containing the tube function')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['RadiusArrayName','radiusarray','str',1,'','name of the array where radius values are stored'],
            ['Image','image','vtkImageData',1,'','the input image to use as a reference','vmtkimagereader'],
            ['SampleDimensions','dimensions','int',3,'(0,)','dimensions of the output image'],
            ['ModelBounds','bounds','float',6,'(0.0,)','model bounds in physical coordinates (if None, they are computed automatically)'],
            ['NegateFunction','negate','bool',1,'','produce a function that is negative inside the polyball']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter']])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if self.RadiusArrayName == None:
            self.PrintError('Error: No radius array name.')

        modeller = vtkvmtk.vtkvmtkPolyBallModeller()
        modeller.SetInputData(self.Surface)
        modeller.SetRadiusArrayName(self.RadiusArrayName)
        modeller.UsePolyBallLineOff()
        if self.Image:
            modeller.SetReferenceImage(self.Image)
        else:
            modeller.SetSampleDimensions(self.SampleDimensions)
            if self.ModelBounds:
                modeller.SetModelBounds(self.ModelBounds)
        modeller.SetNegateFunction(self.NegateFunction)
        modeller.Update()

        self.Image = modeller.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkimagenormalize as norm


def test_normalize_image(aorta_image, compare_images):
    name = __name__ + '_test_normalize_image.mha'
    normer = norm.vmtkImageNormalize()
    normer.Image = aorta_image
    normer.Execute()

    assert compare_images(normer.Image, name) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfaceconnectivity.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkSurfaceConnectivity(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.ReferenceSurface = None

        self.ClosestPoint = None

        self.Method = 'largest'

        self.GroupId = -1
        self.GroupIdsArrayName = 'GroupIds'

        self.CleanOutput = 0

        self.SetScriptName('vmtksurfaceconnectivity')
        self.SetScriptDoc('extract the largest connected region, the closest point-connected region, the scalar-connected region from a surface, or all connected regions of a surface tagged with an id')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['Method','method','str',1,'["largest","closest", "all"]','connectivity method'],
            ['ClosestPoint','closestpoint','float',3,'','coordinates of the closest point'],
            ['ReferenceSurface','r','vtkPolyData',1,'','the reference surface, whose barycenter will be used as closest point for the connectivity filter','vmtksurfacereader'],
            ['CleanOutput','cleanoutput','bool',1,'','clean the unused points in the output'],
            ['GroupIdsArrayName','groupidsarray','str',1,'','name of the array containing the connectivity scalar'],
            ['GroupId','groupid','int',1,'','value of the connectivity scalar']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if (self.GroupId != -1) and (self.GroupIdsArrayName!=''):
            self.Surface.GetPointData().SetActiveScalars(self.GroupIdsArrayName)

        barycenter = [0.0,0.0,0.0]
        if self.Method == 'closest' and self.ClosestPoint == None:
            n = self.ReferenceSurface.GetNumberOfPoints()
            for i in range(n):
                point = self.ReferenceSurface.GetPoint(i)
                barycenter[0] += point[0]
                barycenter[1] += point[1]
                barycenter[2] += point[2]
            barycenter[0] /= n
            barycenter[1] /= n
            barycenter[2] /= n

        connectivityFilter = vtk.vtkPolyDataConnectivityFilter()
        connectivityFilter.SetInputData(self.Surface)
        connectivityFilter.ColorRegionsOff()
        if self.Method == 'largest':
            connectivityFilter.SetExtractionModeToLargestRegion()
        elif self.Method == 'closest':
            connectivityFilter.SetExtractionModeToClosestPointRegion()
            if self.ClosestPoint:
                connectivityFilter.SetClosestPoint(self.ClosestPoint)
            else:
                connectivityFilter.SetClosestPoint(barycenter)
        elif self.Method == 'all':
            connectivityFilter.SetExtractionModeToAllRegions()
            connectivityFilter.ColorRegionsOn()

        if self.GroupId != -1:
            connectivityFilter.ScalarConnectivityOn()
            scalarRange = [self.GroupId,self.GroupId]
            connectivityFilter.SetScalarRange(scalarRange)
        connectivityFilter.Update()

        self.Surface = connectivityFilter.GetOutput()

        if self.CleanOutput == 1:
            cleaner = vtk.vtkCleanPolyData()
            cleaner.SetInputConnection(connectivityFilter.GetOutputPort())
            cleaner.Update()

            self.Surface = cleaner.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!${PYTHON_SHEBANG}

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import os
import shutil
home = os.path.expanduser("~")

VERSION = "1.3"

if __name__ == '__main__':

    if sys.platform == "win32":
        try:
            import pkg_resources
            package_path = pkg_resources.get_distribution("vmtk").location
        except:
            package_path = [f for f in sys.path if 'Python27\\lib\\site-packages' in f][0]
        vmtk_exe_path = os.path.join(package_path,"vmtk","bin","vmtk-exe.py")
        DESKTOP_FOLDER = get_special_folder_path("CSIDL_DESKTOPDIRECTORY")
        target = vmtk_exe_path
        description = "vmtk-launcher"
        filename = "vmtk-%s.lnk" % VERSION
        iconpath = os.path.join(package_path,"vmtk","bin","vmtk-icon.ico")
        create_shortcut(target,description,filename,'','',iconpath)
        #move shortcut to Desktop
        shutil.move(os.path.join(os.getcwd(),filename),os.path.join(DESKTOP_FOLDER,filename))
        file_created(os.path.join(DESKTOP_FOLDER,filename))
